/**************************************************************

attach.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "comfun.h"
#include "trajd.h"
#include "console.h"
#include "image.h"
#include "cmpack_common.h"
#include "phot.h"
#include "ccdfile.h"
#include "phfun.h"
#include "attach.h"

int Attach(CmpackPhot *kc, CmpackPhotFrame *frame, CmpackCcdFile *file)
/* Reads lc->image data from fits file into memory buffer */
{
	int nx, ny;
	unsigned mask;
	CmpackCcdParams params;

	cmpack_free(frame->filter);
	frame->filter = NULL;
	cmpack_free(frame->object);
	frame->object = NULL;
	frame->exptime = 0.0;
	frame->ccdtemp = INVALID_TEMP;
	frame->frsum = frame->fravg = 1;
	if (frame->image)
		cmpack_image_destroy(frame->image);
	frame->image = NULL;

    /* Read frame attributes */
	mask = CMPACK_CM_IMAGE | CMPACK_CM_JD | CMPACK_CM_FILTER | CMPACK_CM_OBJECT | 
		CMPACK_CM_EXPOSURE | CMPACK_CM_CCDTEMP | CMPACK_CM_SUBFRAMES;
	if (cmpack_ccd_get_params(file, mask, &params)!=0) {
		printout(kc->con, 0, "Failed to read image parameters from the source file.");
		return CMPACK_ERR_READ_ERROR;
	}
	frame->jd = params.jd;
	if (frame->jd<=0.0) {
		printout(kc->con, 0, "Invalid date and time of observation in the source file");
		return CMPACK_ERR_INVALID_DATE;
	}
	nx = params.image_width;
	ny = params.image_height;
	if (nx<=0 || nx>=65536 || ny<=0 || ny>=65536) {
		printout(kc->con, 0, "Invalid dimensions of the source image");
		return CMPACK_ERR_INVALID_SIZE;
	}
	if (params.filter)
		frame->filter = cmpack_strdup(params.filter);
	if (params.object.designation)
		frame->object = cmpack_strdup(params.object.designation);
	frame->exptime = params.exposure;
	frame->ccdtemp = params.ccdtemp;
	frame->fravg = params.subframes_avg;
	if (frame->fravg<=0)
		frame->fravg = 1;
	frame->frsum = params.subframes_sum;
	if (frame->frsum<=0)
		frame->frsum = 1;

	/* Compute JD for the center of exposure */
    frame->jd += 0.5*frame->exptime/86400.0;

	/* Read image data */
	return cmpack_ccd_to_image(file, CMPACK_BITPIX_DOUBLE, &frame->image);
}
