/**************************************************************

matfun.h (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_MATCH_H
#define CMPACK_MATCH_H

#include "cmpack_match.h"
#include "console.h"
#include "matstack.h"

/* Matching context */
struct _CmpackMatch
{
	int refcnt;					/**< Reference counter */
	CmpackConsole *con;			/**< Output console */

	/* Configuration parameters: */
	int nstar;                  /**< Number of vertices of polygons */
	int maxstar;                /**< Maximum number of stars used in Solve procedure */
	double clip;                /**< Clipping threshold in sigmas */
	CmpackMatchMethod method;	/**< Matching method */
	double maxoffset;			/**< Max. position offset for Simple method */

	/* Reference frame: */
	int width, height;			/**< Frame dimensions in pixels */
	int c1;						/**< No. of stars in reference file */
	double *x1, *y1;			/**< Coordinates of stars */

	/* Last frame */
	int matched_stars;			/**< Number of stars matched */
	double offset[2];			/**< Frame offset in pixels */
};

struct _CmpackMatchObject
{
	int	i;						/**< Object's ordinal number */
	double r;					/**< Relative radius */
};

struct _CmpackMatchFrame
{
	/* Reference frame: */
	int width1, height1;		/**< Reference frame dimensions in pixels */
	int c1;						/**< No. of stars in reference file */
	int n1;						/**< No. of stars for matching */
	double *x1, *y1;			/**< Coordinates of stars */

	/* Input frame: */
	int width2, height2;		/**< Source frame dimensions in pixels */
	int c2;						/**< No. of stars in input file */
	int n2;                     /**< No. of stars for matching */
	int *i2;					/**< Cross-reference id */
	double *x2, *y2;   			/**< Coordinates of stars */

	/* Preallocated buffers: */
	struct _CmpackMatchObject *idr1, *idr2;
	int *id1, *id2;
	double *k2, *dev;

	/* Transformation stack: */
	CmpackMatchStack stack;		/**< Transformation stack */

	/* Result */
	int mstar;					/**< Number of matched stars */
	CmpackMatrix trafo;			/**< Transformation matrix */
};
typedef struct _CmpackMatchFrame CmpackMatchFrame;

#endif
