/**************************************************************

matread.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/
#define _NO_OLDNAMES

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "comfun.h"
#include "cmpack_common.h"
#include "cmpack_phtfile.h"
#include "cmpack_catfile.h"
#include "matread.h"

void ReadRef(CmpackMatch *lc, CmpackPhtFile *pht)
{
	int	i, j, count;
	CmpackPhtInfo info;
	CmpackPhtObject obj;

	if (cmpack_pht_get_info(pht, CMPACK_PI_FRAME_PARAMS, &info)==0) {
		lc->width = info.width;
		lc->height = info.height;
	} else {
		lc->width = lc->height = 0;
	}

	cmpack_free(lc->x1);
	cmpack_free(lc->y1);
	lc->x1 = lc->y1 = NULL;
	j = 0;
	count = cmpack_pht_object_count(pht);
	if (count>0) {
		lc->x1 = (double*) cmpack_malloc(count*sizeof(double));
		lc->y1 = (double*) cmpack_malloc(count*sizeof(double));
		for (i=0; i<count; i++) {
			cmpack_pht_get_object(pht, i, CMPACK_PO_CENTER, &obj);
			lc->x1[j] = obj.x;
			lc->y1[j] = obj.y;
			j++;
		}
	}
	/* Normal return */
	lc->c1 = j;
}

void ReadCat(CmpackMatch *lc, CmpackCatFile *cat)
{
	int	i, j, count;
	CmpackCatObject obj;

	lc->width = cmpack_cat_get_width(cat);
	lc->height = cmpack_cat_get_height(cat);

	j = 0;
	cmpack_free(lc->x1);
	cmpack_free(lc->y1);
	lc->x1 = lc->y1 = NULL;
	count = cmpack_cat_nstar(cat);
	if (count>0) {
		lc->x1 = (double*) cmpack_malloc(count*sizeof(double));
		lc->y1 = (double*) cmpack_malloc(count*sizeof(double));
		for (i=0; i<count; i++) {
			if (cmpack_cat_get_star(cat, i, CMPACK_OM_CENTER, &obj)==0) {
				lc->x1[j] = obj.center_x;
				lc->y1[j] = obj.center_y;
				j++;
			}
		}
	}
	/* Normal return */
	lc->c1 = j;
}

void ReadSrc(CmpackMatchFrame *lc, CmpackPhtFile *pht)
{
	int		i, j, count;
	CmpackPhtInfo info;
	CmpackPhtObject obj;

	if (cmpack_pht_get_info(pht, CMPACK_PI_FRAME_PARAMS, &info)==0) {
		lc->width2 = info.width;
		lc->height2 = info.height;
	} else {
		lc->width2 = lc->height2 = 0;
	}

	j = 0;
	cmpack_free(lc->i2);
	cmpack_free(lc->x2);
	cmpack_free(lc->y2);
	lc->i2 = NULL;
	lc->x2 = lc->y2 = NULL;
	count = cmpack_pht_object_count(pht);
	if (count>0) {
		lc->i2 = (int*) cmpack_malloc(count*sizeof(int));
		lc->x2 = (double*) cmpack_malloc(count*sizeof(double));
		lc->y2 = (double*) cmpack_malloc(count*sizeof(double));
		for (i=0; i<count; i++) {
			cmpack_pht_get_object(pht, i, CMPACK_PO_CENTER, &obj);
			lc->x2[j] = obj.x;
			lc->y2[j] = obj.y;
			j++;
		}
	}
	/* Normal return */
	lc->c2 = j;
}
