/**************************************************************

phot.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "comfun.h"
#include "console.h"
#include "cmpack_common.h"
#include "cmpack_phot.h"
#include "phot.h"
#include "attach.h"
#include "find.h"
#include "sky.h"
#include "phwrite.h"
#include "fotometr.h"
#include "sort.h"
#include "phfun.h"

/* Max. no. of pixels used in sky level computation */
#define MAXSKY   10000           
	
/* Min. no. of pixels used in sky level computation */
#define MINSKY   20              

/***************   PUBLIC FUNCTIONS   ********************************/

/* Context initialization */
CmpackPhot *cmpack_phot_init(void)
{
	CmpackPhot *lc = (CmpackPhot*)cmpack_calloc(1, sizeof(CmpackPhot));
	lc->refcnt = 1;
	lc->readns = 15.0;
	lc->gain   = 2.3;
	lc->datalo = 0.0;
	lc->datahi = 65535.0;
	lc->fwhm   = 2.5;
	lc->thresh = 5.0;
	lc->shrplo = 0.2;
	lc->shrphi = 1.0;
	lc->rndlo  = -1.0;
	lc->rndhi  = 1.0;
	lc->ap[0]  = 2.000;
	lc->ap[1]  = 2.558;
	lc->ap[2]  = 3.272;
	lc->ap[3]  = 4.186;
	lc->ap[4]  = 5.354;
	lc->ap[5]  = 6.849;
	lc->ap[6]  = 8.761;
	lc->ap[7]  = 11.206;
	lc->ap[8]  = 14.334;
	lc->ap[8]  = 18.335;
	lc->ap[10] = 23.453;
	lc->ap[11] = 30.000;
	lc->is     = 20.0;
	lc->os     = 30.0;
	return lc;
}

/* Increment the reference counter */
CmpackPhot *cmpack_phot_reference(CmpackPhot *ctx)
{
	ctx->refcnt++;
	return ctx;
}

/* Decrement reference counter / detroy the instance */
void cmpack_phot_destroy(CmpackPhot *ctx)
{
	ctx->refcnt--;
	if (ctx->refcnt==0) {
		if (ctx->con) 
			cmpack_con_destroy(ctx->con);
		cmpack_free(ctx);
	}
}

/* Attach console */
void cmpack_phot_set_console(CmpackPhot *ctx, CmpackConsole *con)
{
	if (con!=ctx->con) {
		if (ctx->con) 
			cmpack_con_destroy(ctx->con);
		ctx->con = con;
		if (ctx->con) 
			cmpack_con_reference(ctx->con);
	}
}

/* Set output image data format */
void cmpack_phot_set_rnoise(CmpackPhot *lc, double rnoise)
{
	lc->readns = rnoise;
}

/* Set output image data format */
double cmpack_phot_get_rnoise(CmpackPhot *lc)
{
	return lc->readns;
}

/* Set output image data format */
void cmpack_phot_set_adcgain(CmpackPhot *lc, double adcgain)
{
	lc->gain = adcgain;
}

/* Set output image data format */
double cmpack_phot_get_adcgain(CmpackPhot *lc)
{
	return lc->gain;
}

/* Set output image data format */
void cmpack_phot_set_fwhm(CmpackPhot *lc, double fwhm)
{
	lc->fwhm = fwhm;
}

/* Set output image data format */
double cmpack_phot_get_fwhm(CmpackPhot *lc)
{
	return lc->fwhm;
}

/* Set output image data format */
void cmpack_phot_set_thresh(CmpackPhot *lc, double thresh)
{
	lc->thresh = thresh;
}

/* Set output image data format */
double cmpack_phot_get_thresh(CmpackPhot *lc)
{
	return lc->thresh;
}

/* Set output image data format */
void cmpack_phot_set_minshrp(CmpackPhot *lc, double minshrp)
{
	lc->shrplo = minshrp;
}

/* Set output image data format */
double cmpack_phot_get_minshrp(CmpackPhot *lc)
{
	return lc->shrplo;
}

/* Set output image data format */
void cmpack_phot_set_maxshrp(CmpackPhot *lc, double maxshrp)
{
	lc->shrphi = maxshrp;
}

/* Set output image data format */
double cmpack_phot_get_maxshrp(CmpackPhot *lc)
{
	return lc->shrphi;
}

/* Set output image data format */
void cmpack_phot_set_minrnd(CmpackPhot *lc, double minrnd)
{
	lc->rndlo = minrnd;
}

/* Set output image data format */
double cmpack_phot_get_minrnd(CmpackPhot *lc)
{
	return lc->rndlo;
}

/* Set output image data format */
void cmpack_phot_set_maxrnd(CmpackPhot *lc, double maxrnd)
{
	lc->rndhi = maxrnd;
}

/* Set output image data format */
double cmpack_phot_get_maxrnd(CmpackPhot *lc)
{
	return lc->rndhi;
}

/* Set output image data format */
void cmpack_phot_set_minval(CmpackPhot *lc, double minval)
{
	lc->datalo = minval;
}

/* Set output image data format */
double cmpack_phot_get_minval(CmpackPhot *lc)
{
	return lc->datalo;
}

/* Set output image data format */
void cmpack_phot_set_maxval(CmpackPhot *lc, double maxval)
{
	lc->datahi = maxval;
}

/* Set output image data format */
double cmpack_phot_get_maxval(CmpackPhot *lc)
{
	return lc->datahi;
}

/* Set image border */
void cmpack_phot_set_border(CmpackPhot *lc, const CmpackBorder *border)
{
	if (border)
		lc->border = *border;
	else
		memset(&lc->border, 0, sizeof(CmpackBorder));
}

/* Set image border */
void cmpack_phot_get_border(CmpackPhot *lc, CmpackBorder *border)
{
	if (border)
		*border = lc->border;
}

/* Set output image data format */
void cmpack_phot_set_skyin(CmpackPhot *lc, double skyin)
{
	lc->is = skyin;
}

/* Set output image data format */
double cmpack_phot_get_skyin(CmpackPhot *lc)
{
	return lc->is;
}

/* Set output image data format */
void cmpack_phot_set_skyout(CmpackPhot *lc, double skyout)
{
	lc->os = skyout;
}

/* Set output image data format */
double cmpack_phot_get_skyout(CmpackPhot *lc)
{
	return lc->os;
}

/* Set output image data format */
void cmpack_phot_set_aper(CmpackPhot *lc, const double *items, int count)
{
	int i;
	double x;

	memset(lc->ap, 0, MAXAP*sizeof(double));
	if (count>MAXAP)
		count = MAXAP;
	for (i=0; i<count; i++) {
		x = items[i];
		lc->ap[i] = ((x>=1 && x<65536) ? x : 0);
	}
}

/* Set output image data format */
void cmpack_phot_get_aper(CmpackPhot *lc, double **items, int *count)
{
	*items = (double*)cmpack_malloc(MAXAP*sizeof(double));
	memcpy(*items, lc->ap, MAXAP*sizeof(double));
	*count = MAXAP;
}

/* Make photometry of 'scifile' to 'outfile'. Returns Julian date and number of stars */
int cmpack_phot(CmpackPhot *lc, CmpackCcdFile *infile, CmpackPhtFile *outfile, int *nstars)
{
	char  msg[MAXLINE];
	int   i, res = 0;
	CmpackPhotFrame frame;

	if (nstars)
		*nstars = 0;
	if (outfile)
		cmpack_pht_clear(outfile);

	/* Print configuration parameters */
	if (is_debug(lc->con)) {
		printout(lc->con, 1, "Photometry context:");
		printpard(lc->con, "Readout noise", 1, lc->readns, 2);
		printpard(lc->con, "ADC gain", 1, lc->gain, 2);
		printpard(lc->con, "Min value", 1, lc->datalo, 2);
		printpard(lc->con, "Max value", 1, lc->datahi, 2);
		printpard(lc->con, "FWHM", 1, lc->fwhm, 2);
		printpard(lc->con, "Threshold", 1, lc->thresh, 2);
		printpard(lc->con, "Min sharpness", 1, lc->shrplo, 2);
		printpard(lc->con, "Max sharpness", 1, lc->shrphi, 2);
		printpard(lc->con, "Min roundness", 1, lc->rndlo, 2); 
		printpard(lc->con, "Max roundness", 1, lc->rndhi, 2);
		printparvd(lc->con, "Apertures", 1, MAXAP, lc->ap, 4);
		printpard(lc->con, "Inner sky radius", 1, lc->is, 2);
		printpard(lc->con, "Outer sky radius", 1, lc->os, 2);
		printparvi(lc->con, "Border", 1, 4, (int*)(&lc->border));
	}

	memset(&frame, 0, sizeof(CmpackPhotFrame));
	frame.naper = 0;
	frame.border = lc->border;
	for (i=0; i<MAXAP; i++) {
		if (lc->ap[i]>=1.0) {
			frame.apid[frame.naper] = i+1;
			frame.aper[frame.naper] = lc->ap[i];
			frame.naper++;
		}
	}
	frame.datalo = lc->datalo;
	frame.datahi = lc->datahi;

	/* Check parameters */
	if (frame.naper==0) {
		printout(lc->con, 0, "There are no valid apertures defined");
		return CMPACK_ERR_INVALID_PAR;
	}

	/* Check source file */
    if (!infile) {
        printout(lc->con, 0, "Invalid CCD frame context");
 	    return CMPACK_ERR_INVALID_PAR;
    }

	/* Check photometry file */
	if (!outfile) {
        printout(lc->con, 0, "Invalid photometry file context");
 	    return CMPACK_ERR_INVALID_PAR;
    }

    /* Attach the image */
    res = Attach(lc, &frame, infile);
    if (res==0) {
        sprintf(msg,"JD (UT)      : %.6f", frame.jd);
        printout(lc->con, 1, msg);
        if (frame.fravg>1) {
        	sprintf(msg,"Frames avgd  : %d", frame.fravg);
        	printout(lc->con, 1, msg);
        }
        if (frame.fravg>1) {
        	sprintf(msg,"Frames sumd  : %d", frame.frsum);
        	printout(lc->con, 1, msg);
        }
    }

    /* Sky */
    if (res==0) {
        res = Sky(&frame);
        if (res==0) {
	        sprintf(msg,"Sky value    : %.1f +- %.1f ADU", frame.skymod, frame.skysig);
	        printout(lc->con, 1, msg);
        }
    }

    /* Find stars */
    if (res==0) {
        res = Find(lc, &frame);
        if (res==0) {
	        sprintf(msg,"Rel. error   : %.2f", frame.relerr);
	        printout(lc->con, 1, msg);
            sprintf(msg,"Stars found  : %d", frame.nstar);
            printout(lc->con, 1, msg);
        }
    }

    /* Aperture photometry */
    if (res==0) {
	    res = PhotSB(lc, &frame, MINSKY, MAXSKY);
        if ((res==0)&&(frame.maglim<99.0)&&(frame.magsq<9.0)) {
            sprintf(msg,"Mag. limit   : %.2f +- %.2f per star in aperture #1.", frame.maglim, frame.magsq);
            printout(lc->con, 1, msg);
            sprintf(msg,"Mean FWHM    : %.2f +- %.2f pixels.", frame.fwhm_med, frame.fwhm_err);
            printout(lc->con, 1, msg);
        }
    }

    /* Sort of file according to key(s) */
    if (res==0) {
	    res = Sorter(&frame);
    }

    /* Write output to photometry file */
    if (res==0) {
	    res = SrtWrite(lc, &frame, outfile);
    }

	/* Pass out the number of stars found */
	if (nstars)
		*nstars = frame.nstar;

	/* Normal return */
    ClearStarList(&frame);
	if (frame.image) 
		cmpack_image_destroy(frame.image);
	cmpack_free(frame.object);
	cmpack_free(frame.filter);
    return res;
}
