/**************************************************************

sort.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>

#include "comfun.h"
#include "cmpack_common.h"
#include "cmpack_phot.h"
#include "phot.h"
#include "sort.h"
#include "phfun.h"

#define D(x,y)    d[((x)-1)+(((y)-1)*ncol)]

static int compare_fn(const void *a, const void *b)
{
	double ma = (*(CmpackPhotStar**)a)->apmag[0], mb = (*(CmpackPhotStar**)b)->apmag[0];
	return (ma < mb ? -1 : (ma > mb ? 1 : 0));
}

int Sorter(CmpackPhotFrame *frame)
{
	int 		i;
	CmpackPhotStar **s, *star;

	/* SECTION 2 - Read the input file in line by line, verbatim */
	s = (CmpackPhotStar**) cmpack_malloc(frame->nstar*sizeof(CmpackPhotStar*));
	for (i=0,star=frame->first; star!=NULL; star=star->next) 
		s[i++] = star;

    /* Perform the sort */
	qsort(s,i,sizeof(CmpackPhotStar*),compare_fn);

    /* Rearanges the list of the stars */
    frame->first = frame->last  = NULL;
    for (i=0; i<frame->nstar; i++) {
	    if (frame->last==NULL) 
			frame->first = s[i]; 
	    else 
	        frame->last->next = s[i];
	    frame->last = s[i];
	    s[i]->next = NULL;
    }

    /* Normal return */
    cmpack_free(s);
    return 0;
}
