/**************************************************************

tcurve.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>

#include "comfun.h"
#include "console.h"
#include "cmpack_common.h"
#include "cmpack_tcurve.h"

/********************   PUBLIC FUNCTIONS   **************************/

/* Open output table, start reading sequence */
int cmpack_tcurve(CmpackFrameSet *fset, CmpackTable **ptable, CmpackTCurveFlags flags,
	CmpackConsole *con)
{	
	int		res, frameids, id_column, jd_column, dx_column, dy_column;
	char	msg[MAXLINE];
	CmpackTable *table;
	CmpackFrameInfo frame;

	*ptable = NULL;
	
	frameids = (flags & CMPACK_TCURVE_FRAME_IDS)!=0;

	/* Create table */
	table = cmpack_tab_init(CMPACK_TABLE_TRACKLIST);

	id_column = -1;
	if (frameids)
		id_column = cmpack_tab_add_column_int(table, "FRAME", 0, INT_MAX, -1);

	jd_column = cmpack_tab_add_column_dbl(table, "JD", JD_PRECISION, 1e6, 1e99, 0.0);
	dx_column = cmpack_tab_add_column_dbl(table, "OFFSETX", POS_PRECISION, -1e99, 1e99, 0.0);
	dy_column = cmpack_tab_add_column_dbl(table, "OFFSETY", POS_PRECISION, -1e99, 1e99, 0.0);

	/* Process frames */
	res = cmpack_fset_rewind(fset);
	while (res==0) {
		cmpack_tab_append(table);
		cmpack_fset_get_frame(fset, CMPACK_FI_ID | CMPACK_FI_JULDAT | CMPACK_FI_FILENAME | CMPACK_FI_OFFSET, &frame);
		/* Check frame header */
		if (frameids && frame.frame_id<0) {
			if (frame.filename) 
				sprintf(msg, "%s: Invalid frame identifier", frame.filename);
			else
				strcpy(msg, "Invalid frame identifier");
			printout(con, 0, msg);
		}
		if (frame.juldat<=0) {
			if (frame.filename) 
				sprintf(msg, "%s: Invalid Julian date of observation", frame.filename);
			else
				strcpy(msg, "Invalid Julian date of observation");
			printout(con, 0, msg);
		}
		if (id_column>=0)
			cmpack_tab_ptdi(table, id_column, frame.frame_id);
		if (jd_column>=0)
			cmpack_tab_ptdd(table, jd_column, frame.juldat);
		if (dx_column>=0) 
			cmpack_tab_ptdd(table, dx_column, frame.offset[0]);
		if (dy_column>=0)
			cmpack_tab_ptdd(table, dy_column, frame.offset[1]);
		/* Debug printout */
		if (is_debug(con)) {
			if (frame.filename)
				sprintf(msg, "FRAME: %d (%s)", frame.frame_id, frame.filename);
			else
				sprintf(msg, "FRAME: %d", frame.frame_id);
			printout(con, 1, msg);
			sprintf(msg, "JD: %.7f", frame.juldat); 
			printout(con, 1, msg);
			sprintf(msg, "OFFSET: %.7f %.7f", frame.offset[0], frame.offset[1]); 
			printout(con, 1, msg);
		}
		res = cmpack_fset_next(fset);
	}

	*ptable = table;
	return CMPACK_ERR_OK;
}
