/**************************************************************

amasslist_dlg.h (C-Munipack project)
The 'Plot air mass coefficients' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_AMASSCURVE_DLG_H
#define CMPACK_AMASSCURVE_DLG_H

#include <gtk/gtk.h>

#include "cmpack_widgets.h"
#include "table_class.h"
#include "frameset_class.h"
#include "output_dlg.h"
#include "menubar.h"
#include "popup.h"
#include "infobox.h"
#include "measurement.h"

//
// Air mass coefficients graph
//
class CAMassCurveDlg:public COutputCurveDlg
{
public:
	// Constructor
	CAMassCurveDlg(void);

	// Destructor
	virtual ~CAMassCurveDlg();

	// Initialize the dialog
	bool Make(GtkWindow *parent, bool selected_files, 
		const CObjectCoords &obj, const CLocation &loc);

protected:
	// Get icon name
	virtual const char *GetIconName(void) { return "airmasscurve"; }

	// Frame set was updated
	virtual void OnFrameSetChanged(void);

	// Get list of GtkTreePaths of all selected frames
	virtual GList *GetSelectedFrames(void);

	// Get path to the first selected frame
	virtual GtkTreePath *GetSelectedFrame(void);

private:
	enum tDisplayMode {
		DISPLAY_GRAPH,
		DISPLAY_TABLE
	};

	enum tInfoMode {
		INFO_NONE,
		INFO_STATISTICS,
		INFO_MEASUREMENT
	};

	CMenuBar		m_Menu;
	GtkWidget		*m_ZoomLabel, *m_XLabel, *m_XCombo, *m_YLabel, *m_YCombo;
	GtkWidget		*m_GraphScrWnd, *m_GraphView, *m_TableScrWnd, *m_TableView;
	GtkListStore	*m_XChannels, *m_YChannels;
	GtkToolItem		*m_ZoomFit, *m_ZoomIn, *m_ZoomOut; 
	bool			m_UpdatePos, m_LastPosValid, m_ShowGrid;
	double			m_LastPosX, m_LastPosY;
	int				m_LastFocus;
	int				m_InFiles, m_OutFiles;
	tDisplayMode	m_DispMode;
	tDateFormat		m_DateFormat;
	int				m_ChannelY;
	CObjectCoords	m_ObjCoords;
	CLocation		m_Location;
	CTable			m_Table;
	gint			m_TimerId;
	CPopupMenu		m_GraphMenu;
	CTextBox		m_InfoBox;
	tInfoMode		m_InfoMode;
	CMeasurementBox	m_MeasBox;
	CmpackGraphData	*m_GraphData;
	GtkTreeModel	*m_TableData;

	void UpdateAirMassCurve(void);
	void UpdateChannels(void);
	void UpdateGraphTable(gboolean autozoom_x, gboolean autozoom_y);
	void UpdateStatus(void);
	void UpdateControls(void);
	void UpdateTools(void);
	void EditProperties(void);
	void SetData(int column);
	void SaveData(void);
	void Export(void);
	void SetDisplayMode(tDisplayMode mode);
	void SetInfoMode(tInfoMode mode);
	void PrintValue(char *buf, double val, const CChannel *channel);
	void PrintKeyValue(char *buf, double val, const CChannel *channel);

	void OnCommand(int cmd_id);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnEntryChanged(GtkWidget *pButton);
	void OnContextMenu(GtkWidget *widget, GdkEventButton *event);
	void OnSelectionChanged(void);
	void OnInfoBoxClosed(void);

	static void button_clicked(GtkWidget *pButton, CAMassCurveDlg *pDlg);
	static void entry_changed(GtkWidget *pButton, CAMassCurveDlg *pMe);
	static void mouse_moved(GtkWidget *pGraph, CAMassCurveDlg *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CAMassCurveDlg *pMe);
	static gboolean timer_cb(CAMassCurveDlg *pMe);
	static void selection_changed(GtkWidget *pChart, CAMassCurveDlg *pMe);
	static void MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static void InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
};

//
// Air mass curve options
//
class CAMassCurveOptionsDlg
{
public:
	// Constructor
	CAMassCurveOptionsDlg(GtkWindow *pParent);

	// Destructor
	~CAMassCurveOptionsDlg();

	// Execute the dialog
	bool Execute(CObjectCoords &obj, CLocation &loc);

private:
	GtkWidget		*m_pDlg, *m_OptBox, *m_HelCor, *m_AirMass;
	GtkWidget		*m_ObjBox, *m_ObjName, *m_ObjBtn, *m_RA, *m_Dec;
	GtkWidget		*m_ObjLabel, *m_RALabel, *m_RAUnit, *m_DecLabel, *m_DecUnit;
	GtkWidget		*m_LocBox, *m_LocName, *m_LocBtn, *m_Lon, *m_Lat;
	GtkWidget		*m_LocLabel, *m_LonLabel, *m_LonUnit, *m_LatLabel, *m_LatUnit;
	CObjectCoords	m_ObjCoords;
	CLocation		m_Location;

	void EditObjectCoords(void);
	void EditLocation(void);
	void UpdateObjectCoords(void);
	void UpdateLocation(void);

	bool OnResponseDialog(gint response_id);
	void OnButtonClicked(GtkWidget *pBtn);
	bool OnCloseQuery(void);

	static void response_dialog(GtkDialog *widget, gint response_id, CAMassCurveOptionsDlg *pDlg);
	static void button_clicked(GtkWidget *button, CAMassCurveOptionsDlg *pDlg);
};

#endif
