/**************************************************************

catfile_dlg.h (C-Munipack project)
The preview dialog for catalog files
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_CAT_FILE_DLG_H
#define CMPACK_CAT_FILE_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "menubar.h"
#include "popup.h"
#include "infobox.h"
#include "catalog_class.h"
#include "progress_dlg.h"

//
// Non-modal file viewer
//
class CCatFileDlg:public CFileDlg
{
public:
	// Constructor
	CCatFileDlg(void);

	// Destructor
	virtual ~CCatFileDlg(void);

	// Environment changed, reload settings
	virtual void EnvironmentChanged(void);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath, int *result);

	// Save a file
	virtual bool SaveFile(const char *fpath, int *result);

	// Update dialog controls
	virtual void UpdateControls(void);

private:
	enum tInfoMode {
		INFO_NONE,
		INFO_OBJECT
	};

	CCatalog		m_File;
	GtkWidget		*m_Chart, *m_SelectCbx;
	GtkWidget		*m_NoteBox, *m_NoteCap, *m_NoteApplyBtn, *m_NoteDiscardBtn, *m_NoteIcon;
	GtkToolItem		*m_ClearBtn, *m_SaveBtn, *m_RemoveBtn;
	GtkToolItem		*m_ZoomFit, *m_ZoomIn, *m_ZoomOut;
	GtkListStore	*m_Selections;
	CMenuBar		m_Menu;
	CmpackChartData	*m_ChartData;
	CTextBox		m_InfoBox;
	bool			m_Rulers, m_Negative, m_RowsUpward, m_Updating, m_UpdatePos, m_UpdateZoom;
	bool			m_EditSelection, m_NotSavedBackup, m_SelectionChanged, m_ShowNewSelection;
	int				m_LastPosX, m_LastPosY, m_LastFocus;
	int				m_SelectedStarIndex, m_SelectedStarId;
	int				m_SelectionIndex, m_SelectionIndexBackup;
	tInfoMode		m_InfoMode;
	gint			m_TimerId;
	CPopupMenu		m_ObjectMenu, m_ChartMenu, m_SelectMenu, m_ContextMenu;
	CSelectionList	m_SelectionList, m_SelectionListBackup;
	CSelection		m_CurrentSelection, m_NewSelection;
	CTags			m_Tags, m_TagsBackup;

	void LoadSelectionList(const CSelectionList &list);
	void UpdateSelectionList(void);
	void UpdateChart(void);
	void ExportChart(void);
	void SetInfoMode(tInfoMode mode);
	void ShowProperties(void);
	void EditHeader(void);
	void BeginEditSelection();
	void EndEditSelection(bool apply);
	void UpdateInfoBox(void);
	void UpdateStatus(void);
	void UpdateZoom(void);
	void Select(int index, CmpackSelectionType type);
	void Unselect(int index);
	void UnselectType(CmpackSelectionType type);
	void NewSelection(void);
	void SaveSelection(void);
	void RemoveSelection(void);
	void EditTag(int row, int star_id);
	void SetTag(int row, int star_id, const char *tag);
	void RemoveTag(int row, int star_id);
	void ClearTags(void);
	int ConfirmSelectionChanges();
	void DettachSelection(void);

	void UpdateObject(int row, int star_id);
	void UpdateAll(void);

	void OnCommand(int cmd_id);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnComboChanged(GtkComboBox *widget);
	void OnItemActivated(gint row);
	void OnSelectionChanged(void);
	void OnInfoBoxClosed(void);
	void OnSelectMenu(GdkEventButton *event, gint item);
	void OnObjectMenu(GdkEventButton *event, gint item);
	void OnContextMenu(GdkEventButton *event);
	void OnChartMenu(GdkEventButton *event);
	bool OnNameValidator(const gchar *value, GtkWindow *parent);
	bool OnTagValidator(const gchar *value, GtkWindow *parent);

	static void button_clicked(GtkWidget *pButton, CCatFileDlg *pDlg);
	static void combo_changed(GtkComboBox *widget, CCatFileDlg *pDlg);
	static void mouse_moved(GtkWidget *pChart, CCatFileDlg *pMe);
	static void zoom_changed(GtkWidget *pChart, CCatFileDlg *pMe);
	static void item_activated(GtkWidget *pChart, gint row, CCatFileDlg *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CCatFileDlg *pMe);
	static void selection_changed(GtkWidget *pChart, CCatFileDlg *pMe);
	static gboolean timer_cb(CCatFileDlg *pMe);
	static void MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static void InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static bool name_validator(const gchar *value, GtkWindow *parent, CCatFileDlg *pMe);
	static bool tag_validator(const gchar *value, GtkWindow *parent, CCatFileDlg *pMe);
};

//
// File properties dialog
//
class CCatFileInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CCatFileInfoDlg(GtkWindow *pParent);

	// Display dialog
	void ShowModal(const CCatalog *file, const gchar *name, const gchar *path);

private:
	const CCatalog *m_File;
	const gchar	*m_Name;
	GtkWidget	*m_HdrBtn;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CCatFileInfoDlg *pDlg);
};

//
// Edit header dialog
//
class CCatEditHeaderDlg
{
public:
	// Constructor
	CCatEditHeaderDlg(GtkWindow *pParent);

	// Destructor
	~CCatEditHeaderDlg();

	// Execute the dialog
	bool Execute(CCatalog &file, const gchar *name);

private:
	GtkWidget	*m_pDlg, *m_ObjBtn, *m_LocBtn;
	GtkWidget	*m_ObjName, *m_RA, *m_Dec, *m_Observer, *m_LocName;
	GtkWidget	*m_Lon, *m_Lat, *m_Telescope, *m_Camera, *m_Filter;
	GtkWidget	*m_FOV, *m_Orientation, *m_Notes;

	void SetData(CCatalog &file);
	void GetData(CCatalog &file) const;
	void EditObjectCoords(void);
	void EditLocation(void);

	void OnButtonClicked(GtkWidget *widget);

	static void button_clicked(GtkWidget *widget, CCatEditHeaderDlg *user_data);
};

#endif
