/**************************************************************

stars_dlg.h (C-Munipack project)
The 'Choose stars' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_EDIT_SELECTIONS_DLG_H
#define CMPACK_EDIT_SELECTIONS_DLG_H

#include <gtk/gtk.h>

#include "cmpack_widgets.h"
#include "popup.h"
#include "project.h"
#include "lightcurve_dlg.h"
#include "image_class.h"
#include "phot_class.h"
#include "catalog_class.h"

class CEditSelectionsDlg
{
public:
	// Edit mode
	enum tEditMode { MAKE_LIGHT_CURVE, MAKE_CATALOG_FILE };
	
	// Constructor
	CEditSelectionsDlg(GtkWindow *pParent, tEditMode mode);

	// Destructor
	virtual ~CEditSelectionsDlg();

protected:
	// Display modes
	enum tDisplayMode {
		DISPLAY_IMAGE,
		DISPLAY_CHART,
		DISPLAY_FULL
	};

	tEditMode		m_EditMode;
	GtkWidget		*m_pDlg, *m_Chart, *m_ViewLabel, *m_SelectCbx;
	GtkToolItem		*m_ShowImage, *m_ShowChart, *m_ShowMixed;
	GtkToolItem		*m_ClearBtn, *m_SaveBtn, *m_RemoveBtn;
	GtkToolItem		*m_ZoomIn, *m_ZoomOut, *m_ZoomFit;
	GtkListStore	*m_Selections;
	tDisplayMode	m_DisplayMode;
	CSelectionList	*m_SelectionList;
	CTags			*m_Tags;
	CSelection		m_Current, m_NewSelection;
	CPhot			m_Phot;
	CCatalog		m_Catalog;
	CImage			m_Image;
	CmpackImageData	*m_ImageData;
	CmpackChartData	*m_ChartData;
	CPopupMenu		m_SelectMenu, m_ContextMenu;
	tReferenceType	m_RefType;
	int				m_ApertureIndex, m_SelectionIndex;
	bool			m_Negative, m_RowsUpward, m_InstMagnitudes, m_SingleComparison;
	bool			m_ShowNewSelection, m_Updating;
	
	void UpdateSelectionList(void);
	void UpdateImage(void);
	void UpdateChart(void);
	void UpdateControls(void);
	void Select(int row, int star_id, CmpackSelectionType type);
	void Unselect(int row, int star_id);
	void UnselectType(CmpackSelectionType type);
	void DettachSelection(void);
	void NewSelection(void);
	void SaveSelection(void);
	void RemoveSelection(void);
	void EditTag(int row, int star_id);
	void SetTag(int row, int star_id, const char *tag);
	void RemoveTag(int row, int star_id);
	void ClearTags(void);

	void UpdateObject(int row, int star_id);
	void UpdateAll(void);
	const gchar *NewSelectionName(void);

	void OnButtonClicked(GtkWidget *pButton);
	void OnSelectMenu(GdkEventButton *event, gint item);
	void OnContextMenu(GdkEventButton *event);
	void OnComboChanged(GtkComboBox *widget);
	bool OnResponseDialog(gint response_id);
	bool OnCloseQuery(void);
	bool OnNameValidator(const gchar *value, GtkWindow *parent);
	bool OnTagValidator(const gchar *value, GtkWindow *parent);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CEditSelectionsDlg *pMe);
	static void button_clicked(GtkWidget *pButton, CEditSelectionsDlg *pDlg);
	static void item_activated(GtkWidget *pChart, gint item, CEditSelectionsDlg *pMe);
	static void combo_changed(GtkComboBox *widget, CEditSelectionsDlg *pDlg);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CEditSelectionsDlg *pMe);
	static bool name_validator(const gchar *value, GtkWindow *parent, CEditSelectionsDlg *pMe);
	static bool tag_validator(const gchar *value, GtkWindow *parent, CEditSelectionsDlg *pMe);
};

class CTextQueryDlg
{
public:
	// Validator function prototype
	typedef bool tValidator(const gchar *value, GtkWindow *parent, gpointer data);

public:
	// Constructor
	CTextQueryDlg(GtkWindow *pParent, const gchar *caption);

	// Destructor
	virtual ~CTextQueryDlg();

	// Execute the dialog
	gchar *Execute(const gchar *query, int maxsize, const gchar *defval,
		tValidator *validator, gpointer cb_data);

private:
	GtkWidget	*m_pDlg, *m_Query, *m_Entry;
	tValidator	*m_ValidatorProc;
	gpointer	m_ValidatorData;


	bool OnResponseDialog(gint response_id);
	bool OnCloseQuery(void);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CTextQueryDlg *pMe);
};

#endif
