/**************************************************************

chartexportdlg.h (C-Munipack project)
Export chart dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_EXPORT_DLGS_H
#define CMPACK_EXPORT_DLGS_H

#include <gtk/gtk.h>

class CChartExportDlg
{
public:
	// Constructor
	CChartExportDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CChartExportDlg();

	// Execute the dialog
	bool Execute(CmpackChartData *chart, CmpackImageData *image, const gchar *fileName,
		bool invertImage, bool rowsUpwards);

private:
	GtkWindow			*m_Parent;
	GtkWidget			*m_pDlg;
	class CExportChartOptions *m_Widget;
	int					m_ImageWidth, m_ImageHeight;
	int					m_JpegQuality;
	const gchar			*m_Format;

	gchar *Execute(const gchar *name, int image_width, int image_height);
	
	bool OnResponseDialog(gint response_id);
	void OnTypeChanged(void);
	bool CheckFormat(void);

	static void response_dialog(GtkWidget *widget, gint response_id, CChartExportDlg *pMe);
	static void WidgetCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
};

class CGraphExportDlg
{
public:
	// Constructor
	CGraphExportDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CGraphExportDlg();

	// Execute the dialog
	bool Execute(CmpackGraphView *widget, const gchar *graphName);

private:
	GtkWidget	*m_pDlg;
	GtkWindow	*m_Parent;
	
	bool OnResponseDialog(gint response_id);

	static void response_dialog(GtkWidget *widget, gint response_id, CGraphExportDlg *pMe);
};

class CTableExportDlg
{
public:
	// Constructor
	CTableExportDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CTableExportDlg();

	// Execute the dialog
	void Execute(const CPhot &file, int apertureIndex, const gchar *name,
		int sort_column_id, GtkSortType sort_order);
	void Execute(CTable &table, const gchar *name,
		int sort_column_id, GtkSortType sort_order);

private:
	GtkWidget	*m_pDlg;
	GtkWindow	*m_Parent;

	gchar *Execute(const gchar *name);

	bool OnResponseDialog(gint response_id);

	static void response_dialog(GtkWidget *widget, gint response_id, CTableExportDlg *pMe);
};

class CPhotExportDlg
{
public:
	// Constructor
	CPhotExportDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CPhotExportDlg();

	// Execute the dialog
	void Execute(const CPhot &file, int apertureIndex, const gchar *name,
		int sort_column_id, GtkSortType sort_order);

private:
	GtkWidget	*m_pDlg;
	GtkWindow	*m_Parent;

	gchar *Execute(const gchar *name);

	bool OnResponseDialog(gint response_id);

	static void response_dialog(GtkWidget *widget, gint response_id, CPhotExportDlg *pMe);
};

#endif
