/**************************************************************

lightcurve_dlg.h (C-Munipack project)
The 'Plot light curve' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_LIGHTCURVE_DLG_H
#define CMPACK_LIGHTCURVE_DLG_H

#include <gtk/gtk.h>

#include "cmpack_widgets.h"
#include "output_dlg.h"
#include "frameset_class.h"
#include "table_class.h"
#include "infobox.h"
#include "menubar.h"
#include "output_dlg.h"
#include "progress_dlg.h"
#include "popup.h"
#include "measurement.h"

class CLightCurveDlg:public COutputCurveDlg
{
public:
	struct tParamsRec {
		bool helcor;
		bool amass;
		bool instmag;
	};

	// Constructor
	CLightCurveDlg();

	// Destructor
	virtual ~CLightCurveDlg();

	// Make a light curve dialog
	bool Make(GtkWindow *parent, bool selected_files, const tParamsRec &rec, 
		const CObjectCoords &obj, const CLocation &loc);

	const tParamsRec *Params(void) const
	{ return &m_Params; }
	
protected:
	// Frame set changed, update curve
	virtual void OnFrameSetChanged(void);

	// Get icon name
	virtual const char *GetIconName(void) { return "lightcurve"; }

	// Get list of GtkTreePaths of all selected frames
	virtual GList *GetSelectedFrames(void);

	// Get path to the first selected frame
	virtual GtkTreePath *GetSelectedFrame(void);

private:
	enum tDisplayMode {
		DISPLAY_GRAPH,
		DISPLAY_TABLE
	};

	enum tInfoMode {
		INFO_NONE,
		INFO_STATISTICS,
		INFO_MEASUREMENT
	};

	GtkWidget		*m_XLabel, *m_DCombo, *m_XCombo, *m_YLabel, *m_YCombo, *m_ALabel, *m_ACombo;
	GtkWidget		*m_GraphScrWnd, *m_GraphView, *m_TableScrWnd, *m_TableView;
	GtkWidget		*m_ZoomLabel;
	GtkToolItem		*m_ZoomFit, *m_ZoomIn, *m_ZoomOut; 
	bool			m_UpdatePos, m_LastPosValid, m_ShowErrors, m_ShowGrid;
	tDisplayMode	m_DispMode;
	tDateFormat		m_DateFormat;
	int				m_ApertureIndex, m_ChannelX, m_ChannelY;
	double			m_LastPosX, m_LastPosY;
	int				m_LastFocus;
	tParamsRec		m_Params;
	CMenuBar		m_Menu;
	CObjectCoords	m_ObjCoords;
	CLocation		m_Location;
	CTable			m_Table;
	GtkListStore	*m_DateFormats, *m_XChannels, *m_YChannels, *m_Apertures;
	gint			m_TimerId;
	CPopupMenu		m_GraphMenu;
	CTextBox		m_InfoBox;
	tInfoMode		m_InfoMode;
	CMeasurementBox	m_MeasBox;
	CmpackGraphData	*m_GraphData;
	GtkTreeModel	*m_TableData;
	
	void EditSelection(void);
	void ChangeAperture(void);
	void UpdateLightCurve(void);
	void UpdateChannels(void);
	void UpdateApertures(void);
	void UpdateGraphTable(gboolean autozoom_x, gboolean autozoom_y);
	void UpdateStatus(void);
	void UpdateControls(void);
	void UpdateTools(void);
	void SaveLightCurve(void);
	void EditProperties(void);
	void ShowChart(void);
	void SetDisplayMode(tDisplayMode mode);
	void SetInfoMode(tInfoMode mode);
	void Export(void);
	void PrintValue(char *buf, double val, const CChannel *channel);
	void PrintKeyValue(char *buf, double val, const CChannel *channel);
	void PrintKeyValueU(char *buf, double val, double sdev, const CChannel *channel);

	void OnCommand(int cmd_id);
	void OnMouseMoved(void);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnContextMenu(GtkWidget *widget, GdkEventButton *event);
	void OnEntryChanged(GtkWidget *pButton);
	void OnUpdateCurve(void);
	void OnSelectionChanged(void);
	void OnInfoBoxClosed(void);

	static void button_clicked(GtkWidget *pButton, CLightCurveDlg *pDlg);
	static void mouse_moved(GtkWidget *pGraph, CLightCurveDlg *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CLightCurveDlg *pMe);
	static void entry_changed(GtkWidget *pButton, CLightCurveDlg *pMe);
	static gboolean update_status_timer(CLightCurveDlg *pMe);
	static void selection_changed(GtkWidget *pChart, CLightCurveDlg *pMe);
	static void MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static void InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
};


//
// Light curve options
//
class CLightCurveOptionsDlg
{
public:
	// Constructor
	CLightCurveOptionsDlg(GtkWindow *pParent);

	// Destructor
	~CLightCurveOptionsDlg();

	// Execute the dialog
	bool Execute(CLightCurveDlg::tParamsRec &params, CObjectCoords &obj,
		CLocation &loc);

private:
	GtkWidget		*m_pDlg, *m_OptBox, *m_HelCor, *m_AirMass, *m_InstMag;
	GtkWidget		*m_ObjBox, *m_ObjName, *m_ObjBtn, *m_RA, *m_Dec;
	GtkWidget		*m_ObjLabel, *m_RALabel, *m_RAUnit, *m_DecLabel, *m_DecUnit;
	GtkWidget		*m_LocBox, *m_LocName, *m_LocBtn, *m_Lon, *m_Lat;
	GtkWidget		*m_LocLabel, *m_LonLabel, *m_LonUnit, *m_LatLabel, *m_LatUnit;
	bool			m_ComputeHC, m_ComputeAM, m_ShowInstMag;
	CObjectCoords	m_ObjCoords;
	CLocation		m_Location;

	void EditObjectCoords(void);
	void EditLocation(void);
	void UpdateObjectCoords(void);
	void UpdateLocation(void);
	void UpdateControls(void);

	bool OnResponseDialog(gint response_id);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnButtonToggled(GtkWidget *pBtn);
	bool OnCloseQuery(void);

	static void response_dialog(GtkDialog *widget, gint response_id, CLightCurveOptionsDlg *pDlg);
	static void button_clicked(GtkWidget *button, CLightCurveOptionsDlg *pDlg);
	static void button_toggled(GtkWidget *widget, CLightCurveOptionsDlg *pDlg);
};

//
// Save light curve
//
class CSaveLightCurveDlg
{
public:
	// Constructor
	CSaveLightCurveDlg(GtkWindow *pParent, bool instMag);

	// Destructor
	~CSaveLightCurveDlg();

	// Execute the dialog
	bool Execute(const CTable &table, bool jdhel);

private:
	enum tFileType {
		TYPE_MUNIPACK,
		TYPE_AVE,
		TYPE_MCV,
		TYPE_N_ITEMS			// Number of file formats
	};

	enum tDateType {
		DATE_JD_GEOCENTRIC,
		DATE_JD_HELIOCENTRIC,
		DATE_N_ITEMS			// Number of JD formats
	};
	
	GtkWindow		*m_pParent;
	GtkWidget		*m_pDlg, *m_TypeCombo, *m_JDCombo;
	GtkListStore	*m_FileTypes, *m_JDTypes;
	CTable			m_Table;
	bool			m_Updating, m_InstMag;
	tFileType		m_FileType;
	tDateType		m_JDType;
	gchar			*m_ExportPath;
		
	void OnSelectionChanged(GtkComboBox *pWidget);
	int OnExportCurve(CProgressDlg *sender);
	void OnTypeChanged(void);

	static tFileType StrToFileType(const gchar *str);
	static const gchar *FileTypeToStr(tFileType type);

	static void selection_changed(GtkComboBox *pWidget, CSaveLightCurveDlg *pMe);
	static int ExportLCurveProc(CProgressDlg *sender, void *user_data);
};

#endif
