/**************************************************************

file_dlg.h (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_PHT_FILE_DLG_H
#define CMPACK_PHT_FILE_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "progress_dlg.h"
#include "menubar.h"
#include "infobox.h"
#include "phot_class.h"

//
// Non-modal file viewer
//
class CPhtFileDlg:public CFileDlg
{
public:
	// Constructor
	CPhtFileDlg(void);

	// Destructor
	virtual ~CPhtFileDlg();

	// Environment changed, reload settings
	virtual void EnvironmentChanged(void);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath, int *result);

	// Save a file
	virtual bool SaveFile(const char *fpath, int *result);

	// Update dialog controls
	virtual void UpdateControls(void);

private:
	enum tInfoMode {
		INFO_NONE,
		INFO_OBJECT
	};

	CPhot			m_File;
	GtkWidget		*m_Chart, *m_AperLabel, *m_AperCombo;
	GtkToolItem		*m_ZoomFit, *m_ZoomIn, *m_ZoomOut;
	GtkListStore	*m_Apertures;
	CMenuBar		m_Menu;
	CmpackChartData	*m_ChartData;
	CApertures		m_Aper;
	CTextBox		m_InfoBox;
	bool			m_Updating, m_UpdatePos, m_UpdateZoom;
	bool			m_Negative, m_RowsUpward, m_Rulers;
	int				m_ApertureIndex, m_SelectedIndex, m_SelectedStar;
	int				m_LastPosX, m_LastPosY, m_LastFocus;
	tInfoMode		m_InfoMode;
	guint			m_TimerId;

	void UpdateChart(void);
	void UpdateApertures(void);
	void ExportChart(void);
	void SetInfoMode(tInfoMode mode);
	void ShowProperties(void);
	void UpdateInfoBox(void);
	void UpdateStatus(void);
	void UpdateZoom(void);

	void OnCommand(int cmd_id);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnItemActivated(gint row);
	void OnSelectionChanged(void);
	void OnInfoBoxClosed(void);

	static void button_clicked(GtkWidget *pButton, CPhtFileDlg *pDlg);
	static void mouse_moved(GtkWidget *pChart, CPhtFileDlg *pMe);
	static void zoom_changed(GtkWidget *pChart, CPhtFileDlg *pMe);
	static void item_activated(GtkWidget *pChart, gint row, CPhtFileDlg *pMe);
	static void selection_changed(GtkWidget *pChart, CPhtFileDlg *pMe);
	static gboolean timer_cb(CPhtFileDlg *pMe);
	static void MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static void InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
};


//
// File properties dialog
//
class CPhtFileInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CPhtFileInfoDlg(GtkWindow *pParent);

	// Display dialog
	void ShowModal(const CPhot *info, const gchar *name, const gchar *path);

private:
	const CPhot	*m_File;
	GtkWidget	*m_HdrBtn;
	const gchar	*m_Name;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CPhtFileInfoDlg *pDlg);
};

#endif
