/**************************************************************

proc_classes.h (C-Munipack project)
Process helper classes
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_PROC_CLASSES_H
#define MUNIWIN_PROC_CLASSES_H

#include <gtk/gtk.h>

#include "progress_dlg.h"
#include "helper_classes.h"
#include "frameset_class.h"
#include "table_class.h"

//
// Frame conversion context
//
class CConvertProc
{
public:
	// Constructor
	CConvertProc();

	// Destructor
	~CConvertProc();

	// Initialization
	int Init(CConsole *pCon);

	// Convert file
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CmpackKonv		*m_Konv;
	CmpackBitpix	m_Bitpix;
	CmpackBorder	m_Border;
	CmpackChannel	m_Channel;
	int 			m_FlipV, m_FlipH, m_Binning, m_TimeOffset;
};


//
// Time correction context
//
class CTimeCorrProc
{
public:
	// Constructor
	CTimeCorrProc();

	// Destructor
	~CTimeCorrProc();

	// Initialization
	int Init(CConsole *pCon, double seconds);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CmpackConsole	*m_Console;
	double			m_Seconds;
};


//
// Bias correction context
//
class CBiasCorrProc
{
public:
	// Constructor
	CBiasCorrProc();

	// Destructor
	~CBiasCorrProc();

	// Initialization
	int Init(CConsole *pCon, const gchar *bias);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CmpackBiasCorr	*m_Bias;
	CConsole		*m_Progress;
	gchar			*m_BiasFile;

	// Check temporary bias file
	bool CheckBiasFile(const gchar *orig_file);

	// Convert bias frame
	int ConvertBiasFrame(CConsole *pCon, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Dark correction context
//
class CDarkCorrProc
{
public:
	// Constructor
	CDarkCorrProc();

	// Destructor
	~CDarkCorrProc();

	// Initialization
	int Init(CConsole *pCon, const gchar *dark);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CmpackDarkCorr	*m_Dark;
	gchar			*m_DarkFile;

	// Check temporary dark file
	bool CheckDarkFile(const gchar *orig_file);

	// Convert bias frame
	int ConvertDarkFrame(CConsole *pCon, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Flat correction context
//
class CFlatCorrProc
{
public:
	// Constructor
	CFlatCorrProc();

	// Destructor
	~CFlatCorrProc();

	// Initialization
	int Init(CConsole *pCon, const gchar *flat);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CmpackFlatCorr	*m_Flat;
	gchar			*m_FlatFile;

	// Check temporary dark file
	bool CheckFlatFile(const gchar *orig_file);

	// Convert bias frame
	int ConvertFlatFrame(CConsole *pCon, const gchar *orig_filepath, const gchar *temp_filename);
};


//
// Photometry context
//
class CPhotometryProc
{
public:
	// Constructor
	CPhotometryProc();

	// Destructor
	~CPhotometryProc();

	// Initialization
	int Init(CConsole *pCon);

	// Process file
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CmpackPhot		*m_Phot;
};


//
// Matching context
//
class CMatchingProc
{
public:
	// Constructor
	CMatchingProc();

	// Destructor
	~CMatchingProc();

	// Initialization, change reference frame
	int InitWithReferenceFrame(CConsole *pCon, GtkTreePath *path);

	// Initialization, change reference frame
	int InitWithCatalogFile(CConsole *pCon, const gchar *cat_file);

	// Reinitialization, use current settings
	int Reinitialize(CConsole *pCon);

	// Match file
	int Execute(GtkTreePath *pPath);

private:
	CmpackMatch		*m_Match;
	CConsole		*m_Progress;

	// Check temporary catalogue file
	bool CheckCatalogFile(const gchar *orig_file);

	// Copy catalog photometry and image file
	void ConvertCatFile(const gchar *orig_file, const gchar *temp_file);
};


//
// Master bias context
//
class CMasterBiasProc
{
public:
	// Constructor
	CMasterBiasProc();

	// Destructor
	~CMasterBiasProc();

	// Initialization
	int Open(CConsole *pCon, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CConsole			*m_Progress;
	CmpackMasterBias	*m_MBias;
};


//
// Master dark context
//
class CMasterDarkProc
{
public:
	// Constructor
	CMasterDarkProc();

	// Destructor
	~CMasterDarkProc();

	// Initialization
	int Open(CConsole *pCon, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CConsole			*m_Progress;
	CmpackMasterDark	*m_MDark;
};


//
// Master flat context
//
class CMasterFlatProc
{
public:
	// Constructor
	CMasterFlatProc();

	// Destructor
	~CMasterFlatProc();

	// Initialization
	int Open(CConsole *pCon, const gchar *out_file);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CConsole			*m_Progress;
	CmpackMasterFlat	*m_MFlat;
};


//
// Master flat context
//
class CCombineProc
{
public:
	// Constructor
	CCombineProc();

	// Destructor
	~CCombineProc();

	// Initialization
	int Open(CConsole *pCon, const gchar *out_file, int out_index);

	// Read file
	int Add(GtkTreePath *pPath);

	// Finalization
	int Close(void);

private:
	CConsole		*m_Progress;
	CmpackKombine	*m_Kombine;
	int				m_OutIndex;
};


//
// Make listing
//
class CListProc
{
public:
	// Constructor
	CListProc();

	// Destructor
	~CListProc();

	// Initialization
	int Init(CConsole *pCon, CFrameSet *pFrames, bool raw_frames);

	// Add file
	int Add(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CFrameSet		*m_FrameSet;
	CmpackConsole	*m_Console;
	bool			m_RawFrames;
	int				m_FrameID;

	// Callback function that calls instance's OnPrint
	static void OutputProc(const char *text, void *user_data);

	// Callback function that calls instance's OnPrint
	void Print(const char *text, bool utf8);
};


//
// Munifind class interface
//
class CMFindProc
{
public:
	// Constructor
	CMFindProc();

	// Destructor
	~CMFindProc();

	// Initialization
	int Init(CConsole *pCon, const CAperture &aper, int comp_star);

	// Execute
	int Execute(const CFrameSet &fset, CTable &table, int *comp_star);

private:
	CConsole		*m_Progress;
	CmpackMuniFind	*m_MFind;
};

//
// Light curve
//
int CmpackLightCurve(CConsole *pCon, CTable &table, const CSelection &sel, 
	const CFrameSet &fset, int aperture, CmpackLCurveFlags flags);

//
// Aperture-deviation curve
//
int CmpackApDevCurve(CConsole *pCon, CTable &table, const CSelection &sel, 
	const CFrameSet &fset);

//
// Track curve
//
int CmpackTrackCurve(CConsole *pCon, CTable &table, const CFrameSet &fset); 

//
// Air mass coefficient
//
int CmpackAirMass(CConsole *pCon, CFrameSet &fset, const CObjectCoords &obj, 
	const CLocation &loc);

//
// Air mass curve
//
int CmpackAirMassCurve(CConsole *pCon, CTable &table, const CFrameSet &fset, 
	const CObjectCoords &obj, const CLocation &loc);

//
// Heliocentric correction
//
int CmpackHelCorr(CConsole *pCon, CFrameSet &fset, const CObjectCoords &obj);

//
// Complete reduction of a file
//
class CUpdateProc
{
public:
	// Constructor
	CUpdateProc();

	// Destructor
	~CUpdateProc();

	// Initialization
	int Init(CConsole *pCon);

	// Execute
	int Execute(GtkTreePath *pPath);

private:
	CConsole		*m_Progress;
	CConvertProc	*m_Konv;
	CTimeCorrProc	*m_TCor;
	CBiasCorrProc	*m_Bias;
	CDarkCorrProc	*m_Dark;
	CFlatCorrProc	*m_Flat;
	CPhotometryProc *m_Phot;
	CMatchingProc	*m_Match;
};

#endif
