/**************************************************************

table_dlg.h (C-Munipack project)
The preview dialog for tables
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_TABLE_DLG_H
#define CMPACK_TABLE_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "cmpack_widgets.h"
#include "menubar.h"
#include "infobox.h"
#include "table_class.h"
#include "measurement.h"

//
// Non-modal file viewer
//
class CTableDlg:public CFileDlg
{
public:
	// Constructor
	CTableDlg(void);

	// Destructor
	virtual ~CTableDlg();

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath, int *result);

	// Save a file
	virtual bool SaveFile(const char *fpath, int *result);

	// Update dialog controls
	virtual void UpdateControls(void);

private:
	enum tDisplayMode {
		DISPLAY_GRAPH,
		DISPLAY_TABLE
	};

	enum tInfoMode {
		INFO_NONE,
		INFO_STATISTICS,
		INFO_MEASUREMENT
	};

	GtkWidget		*m_XLabel, *m_XCombo, *m_YLabel, *m_YCombo, *m_ZoomLabel;
	GtkWidget		*m_GraphScrWnd, *m_GraphView, *m_TableScrWnd, *m_TableView;
	GtkToolItem		*m_ZoomFit, *m_ZoomIn, *m_ZoomOut; 
	CMenuBar		m_Menu;
	CTable			m_Table;
	GtkListStore	*m_XChannels, *m_YChannels;
	bool			m_Updating, m_UpdatePos, m_ShowErrors, m_ShowGrid;
	bool			m_LastPosValid;
	int				m_ChannelX, m_ChannelY;
	double			m_LastPosX, m_LastPosY;
	int				m_LastFocus;
	int				m_TimerId;
	CTextBox		m_InfoBox;
	tDisplayMode	m_DispMode;
	tInfoMode		m_InfoMode;
	CMeasurementBox	m_MeasBox;
	CmpackGraphData	*m_GraphData;
	GtkTreeModel	*m_TableData;

	void UpdateChannels(void);
	void UpdateGraph(gboolean autozoom_x, gboolean autozoom_y);
	void UpdateStatus(void);
	void UpdateTools(void);
	void ShowProperties(void);
	void SetDisplayMode(tDisplayMode mode);
	void SetInfoMode(tInfoMode mode);
	void PrintValue(char *buf, double val, const CChannel *channel);
	void PrintKeyValue(char *buf, double val, const CChannel *channel);
	void PrintKeyValueU(char *buf, double val, double sdev, const CChannel *channel);

	void OnCommand(int cmd_id);
	void OnMouseMoved(void);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnChanged(GtkWidget *pButton);
	void OnSelectionChanged(void);
	void OnInfoBoxClosed(void);

	static void button_clicked(GtkWidget *pButton, CTableDlg *pDlg);
	static void mouse_moved(GtkWidget *pGraph, CTableDlg *pMe);
	static void changed(GtkWidget *pButton, CTableDlg *pMe);
	static gboolean timer_cb(CTableDlg *pMe);
	static void selection_changed(GtkWidget *pChart, CTableDlg *pMe);
	static void MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
	static void InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data);
};


//
// File properties dialog
//
class CTableInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CTableInfoDlg(GtkWindow *pParent);

	// Display dialog
	void ShowModal(const CTable *data, const gchar *name, const gchar *path);

private:
	const CTable *m_Table;
	GtkWidget	*m_HdrBtn;
	const gchar	*m_Name;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CTableInfoDlg *pDlg);
};

#endif
