/**************************************************************

varfind.cpp (C-Munipack project)
Widget for the VarFind dialog
Copyright (C) 2011 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>

#include "varfind.h"
#include "configuration.h"
#include "configuration.h"
#include "export_dlgs.h"
#include "lightcurve_dlg.h"
#include "main.h"
#include "proc_classes.h"
#include "utils.h"
#include "choosestars_dlg.h"

enum tCommandId
{
	CMD_HIDE = 100,
	CMD_DELETE,
	CMD_PREVIEW,
	CMD_FRAMEINFO
};

static const CPopupMenu::tPopupMenuItem LCGraphContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_PREVIEW,		"_Show frame" },
	{ CPopupMenu::MB_ITEM, CMD_FRAMEINFO,	"_Show properties" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_HIDE,		"_Delete frames from data set" },
	{ CPopupMenu::MB_ITEM, CMD_DELETE,		"_Remove frames from project" },
	{ CPopupMenu::MB_END }
};

static const CPopupMenu::tPopupMenuItem MDGraphContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_HIDE,		"_Delete objects from data set" },
	{ CPopupMenu::MB_END }
};

static GtkToolItem *toolbar_new_radio(GtkWidget *toolbar, GtkToolItem *first_in_group, 
	const gchar *label, const gchar *tooltip)
{
	GSList *group;
	if (first_in_group)
		group = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(first_in_group));
	else
		group = NULL;

	GtkToolItem *tbtn = gtk_radio_tool_button_new(group);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(tbtn), label);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkWidget *toolbar_new_combo(GtkWidget *toolbar, const gchar *tooltip)
{
	GtkWidget *entry = gtk_combo_box_new();
	gtk_widget_set_size_request(entry, 120, -1);
	gtk_widget_set_tooltip_text(entry, tooltip);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkWidget *toolbar_new_label(GtkWidget *toolbar, const gchar *caption)
{
	GtkWidget *label = gtk_label_new(caption);
	gtk_misc_set_padding(GTK_MISC(label), 8, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	toolbar_new_widget(toolbar, label);
	return label;
}

static GtkWidget *toolbar_new_toolbar(GtkWidget *toolbar)
{
	GtkWidget *bbox = gtk_toolbar_new();
	toolbar_new_widget(toolbar, bbox);
	gtk_toolbar_set_style(GTK_TOOLBAR(bbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_icon_size(GTK_TOOLBAR(bbox), GTK_ICON_SIZE_SMALL_TOOLBAR);
	return bbox;
}

static GtkToolItem *toolbar_new_button(GtkWidget *toolbar, const gchar *label,
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new(NULL, label);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//---------------------------   FIND VARIABLES   --------------------------------

// 
// Default constructor
//
CVarFind::CVarFind(void):m_ChartData(NULL), m_ImageData(NULL), m_MDData(NULL), m_LCData(NULL), 
	m_Image(NULL), m_Phot(NULL), m_Catalog(NULL), m_LCurve(NULL), m_MagDev(NULL), 
	m_ApertureIndex(-1), m_Variable(-1), m_VarIndex(-1), m_VarIndex2(-1), m_Comparison(-1), 
	m_CompIndex(-1), m_SelectMode(SELECT_VARIABLE), m_Updating(false), m_UpdatePos(true), 
	m_Cancelled(false), m_SelectionList(NULL), m_SelectionIndex(-1), m_NewSelectionVar(-1), 
	m_NewSelectionComp(-1), m_ShowNewSelection(false)
{
	GtkWidget *scrwnd;
	GtkTreeIter iter;

	m_DisplayMode = (tDisplayMode)CConfig::GetInt("VarFind", "Mode", DISPLAY_CHART, 0, DISPLAY_MIXED);
	m_DateFormat = (tDateFormat)CConfig::GetInt("Display", "DateFormat", JULIAN_DATE, 0, GREGORIAN_DATE);
	m_Negative = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);
	m_RowsUpward = CConfig::GetBool(CConfig::ROWS_UPWARD);

	// Dialog layout
	m_Box = gtk_table_new(2, 3, FALSE);
	gtk_widget_set_size_request(m_Box, 600, 400); 

	// Varfind-curve graph
	m_MDView = cmpack_graph_view_new();
	cmpack_graph_view_set_activation_mode(CMPACK_GRAPH_VIEW(m_MDView), CMPACK_ACTIVATION_CLICK);
	cmpack_graph_view_set_mouse_control(CMPACK_GRAPH_VIEW(m_MDView), TRUE);
	cmpack_graph_view_set_selection_mode(CMPACK_GRAPH_VIEW(m_MDView), GTK_SELECTION_MULTIPLE);
	g_signal_connect(G_OBJECT(m_MDView), "button_press_event", G_CALLBACK(button_press_event), this);
	g_signal_connect(G_OBJECT(m_MDView), "item-activated", G_CALLBACK(item_activated), this);
	cmpack_graph_view_set_scales(CMPACK_GRAPH_VIEW(m_MDView), TRUE, TRUE);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_MDView);
	gtk_table_attach_defaults(GTK_TABLE(m_Box), scrwnd, 0, 1, 0, 1);

	// Chart
	m_Chart = cmpack_chart_view_new();
	cmpack_chart_view_set_activation_mode(CMPACK_CHART_VIEW(m_Chart), CMPACK_ACTIVATION_CLICK);
	cmpack_chart_view_set_mouse_control(CMPACK_CHART_VIEW(m_Chart), TRUE);
	g_signal_connect(G_OBJECT(m_Chart), "item-activated", G_CALLBACK(item_activated), this);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Chart);
	gtk_table_attach_defaults(GTK_TABLE(m_Box), scrwnd, 1, 2, 0, 1);

	// Light-curve graph
	m_LCView = cmpack_graph_view_new_with_model(NULL);
	cmpack_graph_view_set_mouse_control(CMPACK_GRAPH_VIEW(m_LCView), TRUE);
	cmpack_graph_view_set_scales(CMPACK_GRAPH_VIEW(m_LCView), TRUE, TRUE);
	cmpack_graph_view_set_selection_mode(CMPACK_GRAPH_VIEW(m_LCView), GTK_SELECTION_MULTIPLE);
	g_signal_connect(G_OBJECT(m_LCView), "button_press_event", G_CALLBACK(button_press_event), this);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_LCView);
	gtk_table_attach_defaults(GTK_TABLE(m_Box), scrwnd, 0, 2, 1, 2);
	
	// Upper right toolbar
	GtkWidget *tbar = gtk_toolbar_new();
	gtk_toolbar_set_style (GTK_TOOLBAR (tbar), GTK_TOOLBAR_TEXT);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbar), GTK_ORIENTATION_VERTICAL);
	gtk_table_attach(GTK_TABLE(m_Box), tbar, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_widget_set_size_request(tbar, 160, -1); 

	// Selection mode
	toolbar_new_label(tbar, "Selection");
	m_SelectVar = toolbar_new_radio(tbar, NULL, "Change variable",
		"Switch to selection of a variable star");
	g_signal_connect(G_OBJECT(m_SelectVar), "toggled", G_CALLBACK(button_clicked), this);
	GtkWidget *bbox = toolbar_new_toolbar(tbar);
	m_GoBack = gtk_tool_button_new_from_stock(GTK_STOCK_GO_BACK);
	gtk_widget_set_tooltip_text(GTK_WIDGET(m_GoBack), "Select previous object");
	gtk_toolbar_insert(GTK_TOOLBAR(bbox), m_GoBack, -1);
	g_signal_connect(G_OBJECT(m_GoBack), "clicked", G_CALLBACK(button_clicked), this);
	m_GoForward = gtk_tool_button_new_from_stock(GTK_STOCK_GO_FORWARD);
	gtk_widget_set_tooltip_text(GTK_WIDGET(m_GoForward), "Select next object");
	gtk_toolbar_insert(GTK_TOOLBAR(bbox), m_GoForward, -1);
	g_signal_connect(G_OBJECT(m_GoForward), "clicked", G_CALLBACK(button_clicked), this);
	m_SelectComp = toolbar_new_radio(tbar, m_SelectVar, "Change comparison",
		"Switch to selection of a comparison star");
	g_signal_connect(G_OBJECT(m_SelectComp), "toggled", G_CALLBACK(button_clicked), this);
	toolbar_new_separator(tbar);

	// Saved selections
	m_SelectCbx = toolbar_new_combo(tbar, "Choose an item to restore recently used object selection");
	m_Selections = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_SelectCbx), GTK_TREE_MODEL(m_Selections));
	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_SelectCbx), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_SelectCbx), renderer, "text", 1);
	g_signal_connect(G_OBJECT(m_SelectCbx), "changed", G_CALLBACK(combo_changed), this);
	m_SaveBtn = toolbar_new_button(tbar, "Save as...", "Save the current object selection");
	g_signal_connect(G_OBJECT(m_SaveBtn), "clicked", G_CALLBACK(button_clicked), this);
	m_RemoveBtn = toolbar_new_button(tbar, "Remove", "Remove the current object selection from the list");
	g_signal_connect(G_OBJECT(m_RemoveBtn), "clicked", G_CALLBACK(button_clicked), this);
	toolbar_new_separator(tbar);

	// Aperture selection
	m_AperLabel = toolbar_new_label(tbar, "Aperture");
	m_AperCombo = toolbar_new_combo(tbar, "Aperture used to get brightness of the objects");
	g_signal_connect(G_OBJECT(m_AperCombo), "changed", G_CALLBACK(button_clicked), this);
	m_Apertures = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	GtkCellRenderer *renderer2 = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_AperCombo), renderer2, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_AperCombo), renderer2, "text", 1);
	toolbar_new_separator(tbar);

	// Save buttons
	m_SaveChart = gtk_tool_button_new(NULL, "Save chart");
	gtk_widget_set_tooltip_text(GTK_WIDGET(m_SaveChart), "Save chart to a file");
	gtk_toolbar_insert(GTK_TOOLBAR(tbar), m_SaveChart, -1);
	g_signal_connect(G_OBJECT(m_SaveChart), "clicked", G_CALLBACK(button_clicked), this);

	// Lower right toolbar
	tbar = gtk_toolbar_new();
	gtk_toolbar_set_style (GTK_TOOLBAR (tbar), GTK_TOOLBAR_TEXT);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbar), GTK_ORIENTATION_VERTICAL);
	gtk_table_attach(GTK_TABLE(m_Box), tbar, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

	toolbar_new_label(tbar, "X axis");
	m_XChannels = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	gtk_list_store_append(m_XChannels, &iter);
	gtk_list_store_set(m_XChannels, &iter, 0, JULIAN_DATE, 1, "JD", -1);
	gtk_list_store_append(m_XChannels, &iter);
	gtk_list_store_set(m_XChannels, &iter, 0, GREGORIAN_DATE, 1, "UTC", -1);
	m_XCombo = toolbar_new_combo(tbar, "Column shown on the horizontal axis of the graph");
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_XCombo), GTK_TREE_MODEL(m_XChannels));
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_XCombo), m_DateFormat);
	g_signal_connect(G_OBJECT(m_XCombo), "changed", G_CALLBACK(button_clicked), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_XCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_XCombo), renderer, "text", 1);

	// Save buttons
	m_SaveCurve = gtk_tool_button_new(NULL, "Save light curve");
	gtk_widget_set_tooltip_text(GTK_WIDGET(m_SaveCurve), "Save light curve to a file");
	gtk_toolbar_insert(GTK_TOOLBAR(tbar), m_SaveCurve, -1);
	g_signal_connect(G_OBJECT(m_SaveCurve), "clicked", G_CALLBACK(button_clicked), this);

	// Popup menus
	m_LCGraphMenu.Create(LCGraphContextMenu);
	m_MDGraphMenu.Create(MDGraphContextMenu);
}

//
// Destructor
//
CVarFind::~CVarFind()
{
	// Disconnect graph signals
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_LCView), (gpointer)button_press_event, this);

	// Release objects, free allocated memory
	if (m_MDData)
		g_object_unref(m_MDData);
	if (m_LCData)
		g_object_unref(m_LCData);
	if (m_ChartData)
		g_object_unref(m_ChartData);
	if (m_ImageData)
		g_object_unref(m_ImageData);
	g_object_unref(m_Selections);
}

//
// Button click handler
//
void CVarFind::button_clicked(GtkWidget *pButton, CVarFind *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CVarFind::OnButtonClicked(GtkWidget *pBtn)
{
	int apindex, ch;
	GtkTreeIter iter;

	if (pBtn==m_XCombo) {
		if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(m_XCombo), &iter)) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_XChannels), &iter, 0, &ch, -1);
			if (ch!=m_DateFormat) {
				m_DateFormat = (tDateFormat)ch;
				CConfig::SetInt("Display", "DateFormat", m_DateFormat);
				UpdateLightCurve(FALSE, FALSE);
				UpdateControls();
			}
		}
	} else
	if (pBtn==m_AperCombo) {
		if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(m_AperCombo), &iter)) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_Apertures), &iter, 0, &apindex, -1);
			if (apindex!=m_ApertureIndex) {
				m_ApertureIndex = apindex;
				Callback(CB_APERTURE_CHANGED);
				UpdateChart();
				UpdateControls();
			}
		}
	} else
	if (pBtn==GTK_WIDGET(m_SelectComp)) {
		if (gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(m_SelectComp)))
			m_SelectMode = SELECT_COMPARISON;
	} else
	if (pBtn==GTK_WIDGET(m_SelectVar)) {
		if (gtk_toggle_tool_button_get_active(GTK_TOGGLE_TOOL_BUTTON(m_SelectVar)))
			m_SelectMode = SELECT_VARIABLE;
	} else
	if (pBtn==GTK_WIDGET(m_GoBack)) {
		if (m_MDData && m_VarIndex2>0) {
			int star_id = cmpack_graph_data_get_param(m_MDData, m_VarIndex2-1);
			SetVariableStar(star_id);
			Callback(CB_VARIABLE_CHANGED);
			UpdateControls();
		}
	} else
	if (pBtn==GTK_WIDGET(m_GoForward)) {
		if (m_MDData && m_VarIndex2<cmpack_graph_data_nrows(m_MDData)-1) {
			int star_id = cmpack_graph_data_get_param(m_MDData, m_VarIndex2+1);
			SetVariableStar(star_id);
			Callback(CB_VARIABLE_CHANGED);
			UpdateControls();
		}
	} else
	if (pBtn==GTK_WIDGET(m_SaveChart)) {
		if (m_ChartData && cmpack_chart_data_count(m_ChartData)>0)
			ExportChart();
	} else
	if (pBtn==GTK_WIDGET(m_SaveCurve)) {
		if (m_LCData && cmpack_graph_data_nrows(m_LCData)>0) 
			SaveLightCurve();
	} else
	if (pBtn==GTK_WIDGET(m_SaveBtn)) {
		SaveSelection();
	} else
	if (pBtn==GTK_WIDGET(m_RemoveBtn)) {
		RemoveSelection();
	}
}

//
// Get top level window
//
GtkWindow *CVarFind::GetTopLevel(void)
{
	GtkWidget *toplevel = gtk_widget_get_toplevel(m_Box);
	if (GTK_WIDGET_TOPLEVEL(toplevel) && GTK_IS_WINDOW(toplevel)) 
		return GTK_WINDOW(toplevel);
	return NULL;		
}

//
// Set selection list
//
void CVarFind::SetSelectionList(CSelectionList *list) 
{
	m_SelectionList = list;
	m_SelectionIndex = -1;
	m_ShowNewSelection = true;
	UpdateSelectionList();
	UpdateControls();
}

//
// Update list of selections
//
void CVarFind::UpdateSelectionList(void)
{
	int selectedIndex = 0;
	GtkTreeIter iter;

	m_Updating = true;
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_SelectCbx), NULL);
	gtk_list_store_clear(m_Selections);
	if (m_SelectionList) {
		if (m_ShowNewSelection) {
			gtk_list_store_append(m_Selections, &iter);
			gtk_list_store_set(m_Selections, &iter, 0, -1, 1, "New selection", -1);
		}
		for (int i=0; i<m_SelectionList->Count(); i++) {
			gtk_list_store_append(m_Selections, &iter);
			gtk_list_store_set(m_Selections, &iter, 0, i, 1, m_SelectionList->Name(i), -1);
			if (i == m_SelectionIndex)
				selectedIndex = i;
		}
	} else {
		gtk_list_store_append(m_Selections, &iter);
		gtk_list_store_set(m_Selections, &iter, 0, -1, 1, "New selection", -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_SelectCbx), GTK_TREE_MODEL(m_Selections));
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_SelectCbx), selectedIndex);
	m_Updating = false;
}

void CVarFind::SetSelection(int index, int comp_id, int var_id)
{
	bool var_changed = false, comp_changed = false;
	if (comp_id!=m_Comparison && m_ChartData) {
		if (m_CompIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_CompIndex, NULL);
			cmpack_chart_data_set_color(m_ChartData, m_CompIndex, CMPACK_COLOR_DEFAULT);
			cmpack_chart_data_set_topmost(m_ChartData, m_CompIndex, FALSE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_CompIndex, 0);
		}
		m_Comparison = comp_id;
		m_CompIndex = cmpack_chart_data_find_item(m_ChartData, m_Comparison);
		if (m_CompIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_CompIndex, "comp");
			cmpack_chart_data_set_color(m_ChartData, m_CompIndex, CMPACK_COLOR_GREEN);
			cmpack_chart_data_set_topmost(m_ChartData, m_CompIndex, TRUE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_CompIndex, 4.0);
		}
		comp_changed = true;
	}		
	if (var_id!=m_Variable && m_ChartData && m_MDData) {
		if (m_VarIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_VarIndex, NULL);
			cmpack_chart_data_set_color(m_ChartData, m_VarIndex, CMPACK_COLOR_DEFAULT);
			cmpack_chart_data_set_topmost(m_ChartData, m_VarIndex, FALSE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_VarIndex, 0);
		}
		if (m_VarIndex2>=0) {
			cmpack_graph_data_set_tag(m_MDData, m_VarIndex2, NULL);
			cmpack_graph_data_set_color(m_MDData, m_VarIndex2, CMPACK_COLOR_DEFAULT);
			cmpack_graph_data_set_topmost(m_MDData, m_VarIndex2, FALSE);
		}
		m_Variable = var_id;
		m_VarIndex = cmpack_chart_data_find_item(m_ChartData, m_Variable);
		if (m_VarIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_VarIndex, "var");
			cmpack_chart_data_set_color(m_ChartData, m_VarIndex, CMPACK_COLOR_RED);
			cmpack_chart_data_set_topmost(m_ChartData, m_VarIndex, TRUE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_VarIndex, 4.0);
		}
		m_VarIndex2 = cmpack_graph_data_find_item(m_MDData, m_Variable);
		if (m_VarIndex2>=0) {
			cmpack_graph_data_set_tag(m_MDData, m_VarIndex2, "var");
			cmpack_graph_data_set_color(m_MDData, m_VarIndex2, CMPACK_COLOR_RED);
			cmpack_graph_data_set_topmost(m_MDData, m_VarIndex2, TRUE);
		}
		cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(m_MDView));
		var_changed = true;
	}
	m_SelectionIndex = index;
	if (comp_changed)
		Callback(CB_COMPARISON_CHANGED);
	else if (var_changed)
		Callback(CB_VARIABLE_CHANGED);
	UpdateControls();
}

void CVarFind::DettachSelection(void)
{
	GtkTreeIter iter;

	m_NewSelectionVar = m_Variable;
	m_NewSelectionComp = m_Comparison;
	SetSelection(-1, m_Comparison, m_Variable);
	if (!m_ShowNewSelection) {
		m_ShowNewSelection = true;
		UpdateSelectionList();
	}
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Selections), &iter)) {
		m_Updating = true;
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(m_SelectCbx), &iter);
		m_Updating = false;
	}
}

void CVarFind::SaveSelection(void)
{
	if (!m_SelectionList)
		return;

	const gchar *defValue = (m_SelectionIndex>=0 ? m_SelectionList->Name(m_SelectionIndex) : "");
	CTextQueryDlg dlg(GetTopLevel(), "Save selection as...");
	gchar *name = dlg.Execute("Enter name for the current selection:", 255, defValue, 
		(CTextQueryDlg::tValidator*)name_validator, this);
	if (name) {
		if (m_SelectionIndex<0) 
			m_ShowNewSelection = false;
		else
			m_SelectionList->RemoveAt(m_SelectionIndex);
		CSelection s;
		s.Select(m_Comparison, CMPACK_SELECT_COMP);
		s.Select(m_Variable, CMPACK_SELECT_VAR);
		m_SelectionList->Set(name, s);
		m_SelectionIndex = m_SelectionList->IndexOf(name);
		g_free(name);
		UpdateSelectionList();
		UpdateControls();
	}
}

bool CVarFind::name_validator(const gchar *name, GtkWindow *parent, CVarFind *pMe)
{
	return pMe->OnNameValidator(name, parent);
}

bool CVarFind::OnNameValidator(const gchar *name, GtkWindow *parent)
{
	if (!name || name[0]=='\0') {
		ShowError(parent, "Please, specify name of the selection.");
		return false;
	}
	if (m_SelectionList) {
		int i = m_SelectionList->IndexOf(name);
		if (i>=0 && (m_SelectionIndex<0 || i!=m_SelectionIndex))
			return ShowConfirmation(parent, "A selection with the specified name already exists.\nDo you want to overwrite it?");
	}
	return true;
}

void CVarFind::RemoveSelection(void)
{
	if (!m_SelectionList || m_SelectionIndex<0)
		return;

	m_SelectionList->RemoveAt(m_SelectionIndex);
	if (m_SelectionIndex>=m_SelectionList->Count())
		m_SelectionIndex = m_SelectionList->Count()-1;
	if (m_SelectionIndex<0) {
		m_ShowNewSelection = true;
		m_NewSelectionVar = m_NewSelectionComp = -1;
		SetSelection(-1, m_NewSelectionComp, m_NewSelectionVar);
	} else {
		const CSelection *s = m_SelectionList->At(m_SelectionIndex);
		int v = -1, c = -1;
		for (int i=0; i<s->Count(); i++) {
			if (s->GetType(i) == CMPACK_SELECT_VAR) {
				if (v<0)
					v = s->GetId(i);
			} else if (s->GetType(i) == CMPACK_SELECT_COMP) {
				if (c<0)
					c = s->GetId(i);
			}
		}
		SetSelection(m_SelectionIndex, c, v);
	}
	UpdateSelectionList();
	UpdateControls();
}

//
// Update list of apertures
//
void CVarFind::SetApertures(const CApertures &aper)
{
	char txt[128];
	GtkTreeIter iter;

	m_Aper = aper;
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_AperCombo), NULL);
	gtk_list_store_clear(m_Apertures);
	for (int i=0; i<m_Aper.Count(); i++) {
		const CAperture *ap = m_Aper.Get(i);
		if (ap->Radius()>0)
			sprintf(txt, "#%d (%.2f)", ap->Id(), ap->Radius());
		else
			sprintf(txt, "#%d", ap->Id());
		gtk_list_store_append(m_Apertures, &iter);
		gtk_list_store_set(m_Apertures, &iter, 0, i, 1, txt, -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_AperCombo), GTK_TREE_MODEL(m_Apertures));
	if (m_ApertureIndex<0 || m_ApertureIndex>=m_Aper.Count()) {
		m_ApertureIndex = m_Aper.Find(g_Project->GetInt("LightCurve", "Aperture", 0));
		if (m_ApertureIndex<0)
			m_ApertureIndex = (m_Aper.Count()>0 ? 0 : -1);
	}
	if (m_ApertureIndex>=0) 
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_AperCombo), m_ApertureIndex);
	gtk_widget_set_sensitive(m_AperCombo, m_Aper.Count()>1);
	gtk_widget_set_sensitive(m_AperLabel, m_Aper.Count()>1);

	UpdateControls();
}

void CVarFind::SetImage(CImage *img)
{
	m_Image = img;
	UpdateImage();
	UpdateControls();
}

void CVarFind::UpdateImage(void)
{
	cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), NULL);
	if (m_ImageData) {
		g_object_unref(m_ImageData);
		m_ImageData = NULL;
	}
	if (m_DisplayMode == DISPLAY_IMAGE) {
		m_ImageData = m_Image->ToImageData(m_Negative, false, true, m_RowsUpward);
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), m_ImageData);
	}
}

void CVarFind::SetPhotometryFile(CPhot *phot)
{
	m_Phot = phot;
	m_Catalog = NULL;
	UpdateChart();
	UpdateControls();
}

void CVarFind::SetCatalogFile(CCatalog *cat)
{
	m_Catalog = cat;
	m_Phot = NULL;
	UpdateChart();
	UpdateControls();
}

void CVarFind::UpdateChart(void)
{
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), NULL);
	if (m_ChartData) {
		g_object_unref(m_ChartData);
		m_ChartData = NULL;
	}
	if (m_Phot) {
		m_ApertureIndex = m_Phot->FindAperture(Aperture()->Id());
		m_ChartData = m_Phot->ToChartData(m_ApertureIndex, false, m_DisplayMode==DISPLAY_IMAGE);
	} else
	if (m_Catalog) {
		m_ChartData = m_Catalog->ToChartData(-1, false, m_DisplayMode==DISPLAY_IMAGE);
	}
	if (m_ChartData) {
		int count = cmpack_chart_data_count(m_ChartData);
		for (int i=0; i<count; i++) {
			CmpackChartItem *info = cmpack_chart_data_get_item(m_ChartData, i);
			if (!info->outline && !info->hidden) {
				gboolean retval = TRUE;
				Callback(CB_OBJECT_VALID_QUERY, (int)info->param, &retval);
				if (!retval)
					info->outline = TRUE;
			}
			info->disabled = info->outline;
		}
		m_CompIndex = cmpack_chart_data_find_item(m_ChartData, m_Comparison);
		if (m_CompIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_CompIndex, "comp");
			cmpack_chart_data_set_color(m_ChartData, m_CompIndex, CMPACK_COLOR_GREEN);
			cmpack_chart_data_set_topmost(m_ChartData, m_CompIndex, TRUE);
			if (m_DisplayMode!=DISPLAY_CHART)
				cmpack_chart_data_set_diameter(m_ChartData, m_CompIndex, 4.0);
		}
		m_VarIndex = cmpack_chart_data_find_item(m_ChartData, m_Variable);
		if (m_VarIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_VarIndex, "var");
			cmpack_chart_data_set_color(m_ChartData, m_VarIndex, CMPACK_COLOR_RED);
			cmpack_chart_data_set_topmost(m_ChartData, m_VarIndex, TRUE);
			if (m_DisplayMode!=DISPLAY_CHART)
				cmpack_chart_data_set_diameter(m_ChartData, m_VarIndex, 4.0);
		}
	}
	cmpack_chart_view_set_orientation(CMPACK_CHART_VIEW(m_Chart), m_RowsUpward ? CMPACK_ROWS_UPWARDS : CMPACK_ROWS_DOWNWARDS);
	cmpack_chart_view_set_negative(CMPACK_CHART_VIEW(m_Chart), m_Negative);
	if (m_DisplayMode != DISPLAY_CHART) 
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), m_ImageData);
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), m_ChartData);
}

void CVarFind::SetMagDev(CTable *tab)
{
	m_MagDev = tab;
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_MDView), NULL);
	if (m_MDData)
		g_object_unref(m_MDData);
	m_MDData = m_MagDev->ToGraphData(0, 0, false);
	m_MagDev->SetView(CMPACK_GRAPH_VIEW(m_MDView), 0, 0, false, "mag", "dev");
	m_VarIndex2 = cmpack_graph_data_find_item(m_MDData, m_Variable);
	if (m_VarIndex2>=0) {
		cmpack_graph_data_set_tag(m_MDData, m_VarIndex2, "var");
		cmpack_graph_data_set_color(m_MDData, m_VarIndex2, CMPACK_COLOR_RED);
		cmpack_graph_data_set_topmost(m_MDData, m_VarIndex2, TRUE);
	}
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_MDView), m_MDData);
	cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_MDView), true, true);
	UpdateControls();
}

void CVarFind::SetLightCurve(CTable *tab)
{
	m_LCurve = tab;
	UpdateLightCurve(TRUE, TRUE);
	UpdateControls();
}

void CVarFind::UpdateLightCurve(gboolean autozoom_x, gboolean autozoom_y)
{
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_LCView), NULL);
	if (m_LCData)
		g_object_unref(m_LCData);
	m_LCData = m_LCurve->ToGraphData(0, 0, true);
	m_LCurve->SetView(CMPACK_GRAPH_VIEW(m_LCView), 0, 0, false, NULL, "V-C", m_DateFormat);
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_LCView), m_LCData);
	cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_LCView), autozoom_x, autozoom_y);
}

void CVarFind::SetComparisonStar(int star_id)
{
	if (star_id != m_Comparison && m_ChartData) {
		if (m_CompIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_CompIndex, NULL);
			cmpack_chart_data_set_color(m_ChartData, m_CompIndex, CMPACK_COLOR_DEFAULT);
			cmpack_chart_data_set_topmost(m_ChartData, m_CompIndex, FALSE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_CompIndex, 0);
		}
		DettachSelection();
		if (star_id == m_Variable)
			SetVariableStar(-1);
		m_Comparison = star_id;
		m_CompIndex = cmpack_chart_data_find_item(m_ChartData, star_id);
		if (m_CompIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_CompIndex, "comp");
			cmpack_chart_data_set_color(m_ChartData, m_CompIndex, CMPACK_COLOR_GREEN);
			cmpack_chart_data_set_topmost(m_ChartData, m_CompIndex, TRUE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_CompIndex, 4.0);
		}
	}
}

void CVarFind::SetVariableStar(int star_id)
{
	if (star_id != m_Variable && m_MDData && m_ChartData) {
		// Unselect current variable star 
		if (m_VarIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_VarIndex, NULL);
			cmpack_chart_data_set_color(m_ChartData, m_VarIndex, CMPACK_COLOR_DEFAULT);
			cmpack_chart_data_set_topmost(m_ChartData, m_VarIndex, FALSE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_VarIndex, 0);
		}
		if (m_VarIndex2>=0) {
			cmpack_graph_data_set_tag(m_MDData, m_VarIndex2, NULL);
			cmpack_graph_data_set_color(m_MDData, m_VarIndex2, CMPACK_COLOR_DEFAULT);
			cmpack_graph_data_set_topmost(m_MDData, m_VarIndex2, FALSE);
		}
		// Select new variable star 
		DettachSelection();
		m_Variable = star_id;
		m_VarIndex = cmpack_chart_data_find_item(m_ChartData, star_id);
		if (m_VarIndex>=0) {
			cmpack_chart_data_set_tag(m_ChartData, m_VarIndex, "var");
			cmpack_chart_data_set_color(m_ChartData, m_VarIndex, CMPACK_COLOR_RED);
			cmpack_chart_data_set_topmost(m_ChartData, m_VarIndex, TRUE);
			if (m_DisplayMode==DISPLAY_IMAGE)
				cmpack_chart_data_set_diameter(m_ChartData, m_VarIndex, 4.0);
		}
		m_VarIndex2 = cmpack_graph_data_find_item(m_MDData, m_Variable);
		if (m_VarIndex2>=0) {
			cmpack_graph_data_set_tag(m_MDData, m_VarIndex2, "var");
			cmpack_graph_data_set_color(m_MDData, m_VarIndex2, CMPACK_COLOR_RED);
			cmpack_graph_data_set_topmost(m_MDData, m_VarIndex2, TRUE);
		}
		cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(m_MDView));
	}
}

void CVarFind::UpdateControls(void)
{
	bool ok;
	int var_count;

	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(m_SelectVar), 
		(m_SelectMode == SELECT_VARIABLE));
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(m_SelectComp), 
		(m_SelectMode == SELECT_COMPARISON));

	if (m_SelectMode == SELECT_VARIABLE) {
		gtk_widget_set_sensitive(GTK_WIDGET(m_GoBack), m_VarIndex2>0);
		var_count = (m_MDData ? cmpack_graph_data_nrows(m_MDData) : 0);
		gtk_widget_set_sensitive(GTK_WIDGET(m_GoForward), m_VarIndex2>=0 && m_VarIndex2<var_count-1);
	} else {
		gtk_widget_set_sensitive(GTK_WIDGET(m_GoBack), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(m_GoForward), FALSE);
	}

	ok = EnableCtrlQuery(ID_SAVE_CHART);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SaveChart), ok);

	ok = EnableCtrlQuery(ID_SAVE_LCURVE);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SaveCurve), ok);

	gtk_widget_set_sensitive(m_SelectCbx, m_SelectionList!=NULL);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SaveBtn), m_SelectionList!=NULL && m_VarIndex>=0 && m_CompIndex>=0);
	gtk_widget_set_sensitive(GTK_WIDGET(m_RemoveBtn), m_SelectionList!=NULL && m_SelectionIndex>=0);
}

bool CVarFind::EnableCtrlQuery(tControlId ctrl)
{
	bool ok = false;
	Callback(CB_ENABLE_CTRL_QUERY, ctrl, &ok);
	return ok;
}

void CVarFind::SetDisplayMode(tDisplayMode mode)
{
	m_DisplayMode = mode;
	UpdateImage();
	UpdateChart();
}

//
// Left button click
//
void CVarFind::item_activated(GtkWidget *pWidget, gint item, CVarFind *pDlg)
{
	pDlg->OnItemActivated(pWidget, item);
}

void CVarFind::OnItemActivated(GtkWidget *pWidget, gint item)
{
	int id;

	if (pWidget==m_Chart && m_ChartData) {
		id = (int)cmpack_chart_data_get_param(m_ChartData, item);
		if (m_SelectMode == SELECT_VARIABLE) {
			if (id != m_Variable && id != m_Comparison) {
				SetVariableStar(id);
				Callback(CB_VARIABLE_CHANGED);
				UpdateControls();
			}
		} else {
			if (id != m_Comparison) {
				if (id == m_Variable)
					SetVariableStar(-1);
				SetComparisonStar(id);
				Callback(CB_COMPARISON_CHANGED);
				UpdateControls();
			}
		}
	} else
	if (pWidget == m_MDView && m_MDData) {
		id = (int)cmpack_graph_data_get_param(m_MDData, item);
		if (m_SelectMode == SELECT_VARIABLE) {
			if (id != m_Variable && id != m_Comparison) {
				SetVariableStar(id);
				Callback(CB_VARIABLE_CHANGED);
				UpdateControls();
			}
		} else {
			if (id != m_Comparison) {
				if (id == m_Variable)
					SetVariableStar(-1);
				SetComparisonStar(id);
				Callback(CB_COMPARISON_CHANGED);
				UpdateControls();
			}
		}
	}
}

//
// Save chart to a file
//
void CVarFind::ExportChart(void)
{
	if (m_ChartData) {
		CChartExportDlg dlg(GetTopLevel());
		if (m_DisplayMode==DISPLAY_CHART) 
			dlg.Execute(m_ChartData, NULL, "chart", m_Negative, m_RowsUpward);
		else
			dlg.Execute(m_ChartData, m_ImageData, "chart", m_Negative, m_RowsUpward);
	}
}

//
// Save light curve to file
//
void CVarFind::SaveLightCurve(void)
{
	CSaveLightCurveDlg pDlg(GetTopLevel(), false);
	pDlg.Execute(*m_LCurve, false);
}

//
// Save light curve as image
//
void CVarFind::ExportLightCurve(void)
{
	CGraphExportDlg pDlg(GetTopLevel());
	pDlg.Execute(CMPACK_GRAPH_VIEW(m_LCView), NULL);
}

//
// Save mag-dev curve to a file
//
void CVarFind::SaveMagDevCurve(void)
{
	CSaveMagDevCurveDlg dlg(GetTopLevel());
	dlg.Execute(m_MagDev);
}

//
// Save mag-dev curve as image
//
void CVarFind::ExportMagDevCurve(void)
{
	CGraphExportDlg pDlg(GetTopLevel());
	pDlg.Execute(CMPACK_GRAPH_VIEW(m_MDView), NULL);
}

//
// Mouse button handler
//
gint CVarFind::button_press_event(GtkWidget *widget, GdkEventButton *event, CVarFind *pMe)
{
	int focused;

	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		gtk_widget_grab_focus(widget);
		if (widget==pMe->m_LCView || widget==pMe->m_MDView) {
			focused = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(widget));
			if (focused>=0 && !cmpack_graph_view_is_selected(CMPACK_GRAPH_VIEW(widget), focused)) {
				cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(widget));
				cmpack_graph_view_select(CMPACK_GRAPH_VIEW(widget), focused);
			}
		}
		pMe->OnContextMenu(widget, event);
		return TRUE;
	}
	return FALSE;
}

//
// Show context menu
//
void CVarFind::OnContextMenu(GtkWidget *widget, GdkEventButton *event)
{
	int res, selected;

	if (widget==m_LCView) {
		selected = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_LCView));
		m_LCGraphMenu.Enable(CMD_DELETE, selected>0 && EnableCtrlQuery(ID_DELETE_FROM_PROJECT));
		m_LCGraphMenu.Enable(CMD_HIDE, selected>0 && EnableCtrlQuery(ID_REMOVE_FROM_DATASET));
		m_LCGraphMenu.Enable(CMD_FRAMEINFO, selected==1 && EnableCtrlQuery(ID_SHOW_FRAME_INFO));
		m_LCGraphMenu.Enable(CMD_PREVIEW, selected==1 && EnableCtrlQuery(ID_SHOW_FRAME_PREVIEW));
		res = m_LCGraphMenu.Execute(event);
		switch (res) 
		{
		case CMD_PREVIEW:
			// Show frame preview
			Callback(CB_SHOW_FRAME_PREVIEW);
			break;
		case CMD_FRAMEINFO:
			// Show frame properties
			Callback(CB_SHOW_FRAME_INFO);
			break;
		case CMD_HIDE:
			// Remove selected frames from data set
			Callback(CB_REMOVE_FRAMES_FROM_DATASET);
			break;
		case CMD_DELETE:
			// Delete selected frames from project
			Callback(CB_DELETE_FRAMES_FROM_PROJECT);
			break;
		}
	} else
	if (widget==m_MDView) {
		selected = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_MDView));
		m_LCGraphMenu.Enable(CMD_HIDE, selected>0 && EnableCtrlQuery(ID_REMOVE_FROM_DATASET));
		res = m_MDGraphMenu.Execute(event);
		switch (res) 
		{
		case CMD_HIDE:
			// Remove selected objects from data set
			Callback(CB_REMOVE_OBJECTS_FROM_DATASET);
			break;
		}
	}
}

//
// Get list of indices of selected rows
//
GList *CVarFind::GetSelectedFrames(void) const
{
	return cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_LCView));
}

//
// Get list of indices of selected rows
//
GList *CVarFind::GetSelectedObjects(void) const
{
	return cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_MDView));
}

//
// Get frame ID of first selected row
//
int CVarFind::SelectedFrameID(void) const
{
	int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_LCView));
	if (row>=0 && m_LCData) 
		return (gint)cmpack_graph_data_get_param(m_LCData, row);
	return -1;
}

void CVarFind::combo_changed(GtkComboBox *widget, CVarFind *pDlg)
{
	pDlg->OnComboChanged(widget);
}

void CVarFind::OnComboChanged(GtkComboBox *widget)
{
	GtkTreeIter iter;
	if (widget == GTK_COMBO_BOX(m_SelectCbx)) {
		if (!m_Updating && gtk_combo_box_get_active_iter(widget, &iter)) {
			int index = -1;
			gtk_tree_model_get(GTK_TREE_MODEL(m_Selections), &iter, 0, &index, -1);
			if (index!=m_SelectionIndex) {
				if (index<0) {
					// New selection
					SetSelection(index, m_NewSelectionComp, m_NewSelectionVar);
				} else {
					// Stored selection
					if (m_SelectionIndex<0) {
						m_NewSelectionVar = m_Variable;
						m_NewSelectionComp = m_Comparison;
					}
					const CSelection *s = m_SelectionList->At(index);
					int v = -1, c = -1;
					for (int i=0; i<s->Count(); i++) {
						if (s->GetType(i) == CMPACK_SELECT_VAR) {
							if (v<0)
								v = s->GetId(i);
						} else if (s->GetType(i) == CMPACK_SELECT_COMP) {
							if (c<0)
								c = s->GetId(i);
						}
					}
					SetSelection(index, c, v);
				}
			}
		}
	}
}

//----------------   SAVE MAG-DEV CURVE DIALOG   --------------------------

//
// Constructor
//
CSaveMagDevCurveDlg::CSaveMagDevCurveDlg(GtkWindow *pParent):m_pParent(pParent),
	m_ExportPath(NULL), m_Table(NULL)
{
	m_pDlg = gtk_file_chooser_dialog_new("Save mag-dev curve",
		pParent, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(m_pDlg), true);
}

//
// Destructor
//
CSaveMagDevCurveDlg::~CSaveMagDevCurveDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_free(m_ExportPath);
}

bool CSaveMagDevCurveDlg::Execute(CTable *table)
{
	int res;
	char *filename, *folder;

	m_Table = table;
	g_free(m_ExportPath);
	m_ExportPath = NULL;

	// Restore last folder and file name
	folder = CConfig::GetStr("VarFind", "MagDevFolder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), folder);
	g_free(folder);
	filename = CConfig::GetStr("VarFind", "MagDevFile", "magdevcurve.txt");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return false;

	// Save last folder and file name
	folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(m_pDlg));
	CConfig::SetStr("VarFind", "MagDevFolder", folder);
	g_free(folder);
	m_ExportPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	CConfig::SetStr("VarFind", "MagDevFile", g_basename(m_ExportPath));
	gtk_widget_hide(m_pDlg);

	// Export in separate thread
	CProgressDlg pdlg(m_pParent, "Saving mag-dev curve");
	pdlg.SetFileName(g_basename(m_ExportPath));
	res = pdlg.Execute(ExportProc, this);
	if (res) {
		char *message = cmpack_formaterror(res);
		ShowError(m_pParent, message);
		cmpack_free(message);
		return false;
	}
	return true;
}

int CSaveMagDevCurveDlg::ExportProc(CProgressDlg *sender, void *user_data)
{
	return ((CSaveMagDevCurveDlg*)user_data)->OnExport(sender);
}

int CSaveMagDevCurveDlg::OnExport(CProgressDlg *sender)
{
	bool ok;
	double mag;
	int res = 0, magcol, i;

	CChannels *cx = m_Table->ChannelsX();
	magcol = -1;
	for (i=0; i<cx->Count(); i++) {
		int column = m_Table->ChannelsX()->GetColumn(i);
		if (m_Table->ChannelsX()->GetInfo(i) == DATA_MAGNITUDE) {
			magcol = column;
			break;
		}
	}

	ok = m_Table->Rewind();
	while (ok) {
		if (magcol>=0) 
			m_Table->Show(m_Table->GetDbl(magcol, &mag));
		else
			m_Table->Show(true);
		ok = m_Table->Next();
	}

	m_Table->SaveAs(m_ExportPath, false, false, false, &res);
	return res;
}
