/**************************************************************

varcat.h (C-Munipack project)
Catalogs of variable stars
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_VARFIND_H
#define MUNIWIN_VARFIND_H

#include <gtk/gtk.h>

#include "callback.h"
#include "cmpack_widgets.h"
#include "table_class.h"
#include "phot_class.h"
#include "catalog_class.h"
#include "image_class.h"
#include "progress_dlg.h"
#include "popup.h"

class CVarFind:public CCBObject
{
public:
	// Message codes
	enum tMessageCode {
		CB_APERTURE_CHANGED,
		CB_COMPARISON_CHANGED,
		CB_VARIABLE_CHANGED,
		CB_ENABLE_CTRL_QUERY,
		CB_SHOW_FRAME_PREVIEW,
		CB_SHOW_FRAME_INFO,
		CB_REMOVE_FRAMES_FROM_DATASET,
		CB_DELETE_FRAMES_FROM_PROJECT,
		CB_REMOVE_OBJECTS_FROM_DATASET,
		CB_OBJECT_VALID_QUERY
	};

	// Control identifiers
	enum tControlId
	{
		ID_SAVE_CHART,
		ID_SAVE_LCURVE,
		ID_SHOW_FRAME_PREVIEW,
		ID_SHOW_FRAME_INFO,
		ID_REMOVE_FROM_DATASET,
		ID_DELETE_FROM_PROJECT
	};

	// Display modes
	enum tDisplayMode
	{
		DISPLAY_IMAGE,
		DISPLAY_CHART,
		DISPLAY_MIXED
	};

	CVarFind(void);
	virtual ~CVarFind();

	GtkWidget *Handle(void)
	{ return m_Box; }

	tDisplayMode DisplayMode(void) const
	{ return m_DisplayMode; }

	CmpackGraphData *LightCurveData(void)
	{ return m_LCData; }

	CmpackGraphData *MagDevCurveData(void)
	{ return m_MDData; }

	const CAperture *Aperture(void) const
	{ return m_Aper.Get(m_ApertureIndex); }

	int ApertureIndex(void) const
	{ return m_ApertureIndex; }
	
	int ComparisonID(void) const
	{ return m_Comparison; }

	int VariableID(void) const
	{ return m_Variable; }

	// List of row indices of selected rows
	GList *GetSelectedFrames(void) const;

	// List of column indices of selected objects
	GList *GetSelectedObjects(void) const;

	int SelectedFrameID(void) const;
	
	void SetDisplayMode(tDisplayMode mode);
	void SetImage(CImage *img);
	void SetPhotometryFile(CPhot *phot);
	void SetCatalogFile(CCatalog *file);
	void SetApertures(const CApertures &aper);
	void SetSelectionList(CSelectionList *list);
	void SetMagDev(CTable *tab);
	void SetLightCurve(CTable *tab);
	void SetComparisonStar(int star_id);

	void ExportChart(void);
	void SaveLightCurve(void);
	void SaveMagDevCurve(void);
	void ExportLightCurve(void);
	void ExportMagDevCurve(void);

	void UpdateChart(void);

private:
	// Selection mode
	enum tSelectMode
	{
		SELECT_VARIABLE,
		SELECT_COMPARISON
	};
	
	GtkWidget		*m_Box, *m_MDView, *m_LCView, *m_Chart, *m_SelectCbx;
	GtkToolItem		*m_SelectVar, *m_SelectComp, *m_GoBack, *m_GoForward;
	GtkToolItem		*m_SaveChart, *m_SaveCurve, *m_SaveBtn, *m_RemoveBtn;
	GtkWidget		*m_AperLabel, *m_AperCombo, *m_XCombo;
	GtkListStore	*m_XChannels, *m_Apertures, *m_Selections;
	CmpackChartData	*m_ChartData;
	CmpackImageData	*m_ImageData;
	CmpackGraphData	*m_MDData, *m_LCData;
	CApertures		m_Aper;
	CImage			*m_Image;
	CPhot			*m_Phot;
	CCatalog		*m_Catalog;
	CTable			*m_LCurve, *m_MagDev;
	int				m_ApertureIndex;
	int				m_Variable, m_VarIndex, m_VarIndex2;
	int				m_Comparison, m_CompIndex;
	tSelectMode		m_SelectMode;
	tDisplayMode	m_DisplayMode;
	bool			m_Updating, m_UpdatePos, m_Negative, m_RowsUpward;
	bool			m_Cancelled;
	CPopupMenu		m_LCGraphMenu, m_MDGraphMenu;
	tDateFormat		m_DateFormat;
	CSelectionList	*m_SelectionList;
	int				m_SelectionIndex, m_NewSelectionVar, m_NewSelectionComp;
	bool			m_ShowNewSelection;

	GtkWindow *GetTopLevel(void);
	bool EnableCtrlQuery(tControlId ctrl);
	void SetVariableStar(int star_id);
	void SetSelection(int selectionIndex, int comp_id, int var_id);
	void DettachSelection(void);
	void UpdateImage(void);
	void UpdateLightCurve(gboolean autozoom_x, gboolean autozoom_y);
	void UpdateSelectionList(void);
	void UpdateControls(void);
	void SaveSelection(void);
	void RemoveSelection(void);

	void OnButtonClicked(GtkWidget *pBtn);
	void OnItemActivated(GtkWidget *pChart, gint item);
	void OnContextMenu(GtkWidget *widget, GdkEventButton *event);
	void OnComboChanged(GtkComboBox *widget);
	bool OnNameValidator(const gchar *name, GtkWindow *parent);

	static void button_clicked(GtkWidget *pButton, CVarFind *pMe);
	static void item_activated(GtkWidget *pChart, gint item, CVarFind *pMe);
	static void combo_changed(GtkComboBox *widget, CVarFind *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CVarFind *pMe);
	static bool name_validator(const gchar *name, GtkWindow *parent, CVarFind *pMe);
};

//
// Save mag-dev curve
//
class CSaveMagDevCurveDlg
{
public:
	// Constructor
	CSaveMagDevCurveDlg(GtkWindow *pParent);

	// Destructor
	~CSaveMagDevCurveDlg();

	// Execute the dialog
	bool Execute(CTable *table);

private:
	GtkWindow		*m_pParent;
	GtkWidget		*m_pDlg;
	gchar			*m_ExportPath;
	CTable			*m_Table;

	int OnExport(CProgressDlg *sender);

	static int ExportProc(CProgressDlg *sender, void *user_data);
};

#endif
