/**************************************************************

varfind_dlg.cpp (C-Munipack project)
The 'Find variables' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>

#include "varfindfile_dlg.h"
#include "configuration.h"
#include "main.h"
#include "ctxhelp.h"

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW,
	MENU_HELP
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_REBUILD,
	CMD_SAVE_CHART,
	CMD_SAVE_MAGDEV,
	CMD_SAVE_LCURVE,
	CMD_SHOW_PROPERTIES,
	CMD_SHOW_CHART,
	CMD_SHOW_IMAGE,
	CMD_SHOW_MIXED,
	CMD_EXPORT_DATA,
	CMD_EXPORT_MAGDEV,
	CMD_EXPORT_LCURVE,
	CMD_HELP
};

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_SAVE_MAGDEV,		"Save _mag-dev curve" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_CHART,			"Save _chart" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_LCURVE,		"Save _light curve" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_DATA,		"Export varfind data" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_MAGDEV,		"Export mag-dev curve as image" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_LCURVE,		"Export light curve as image" },
	//{ CMenuBar::MB_ITEM,	CMD_PRINT,				"P_rint" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_SHOW_PROPERTIES,	"Show _Properties" },
	{ CMenuBar::MB_ITEM,	CMD_REBUILD,			"Reload" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_IMAGE,		"_Image only" },
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_CHART,		"_Chart only" },
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_MIXED,		"Image and chart" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_HELP,				"_Show help", "help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu1[] = {
	{ "_File",	MENU_FILE,	FileMenu },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Help",	MENU_HELP,	HelpMenu },
	{ NULL }
};

//-------------------------   VARFIND DIALOG   --------------------------------

//
// Constructor
//
CExtVarFindDlg::CExtVarFindDlg(void):m_PhotometryFile(false), m_ExportPath(NULL)
{
	// Dialog icon
	gchar *icon = get_icon_file("varfind");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Menu bar
	m_Menu.Create(PreviewMenu1, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// VarFind box
	m_VarFind.RegisterCallback(VarFindCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_VarFind.Handle(), TRUE, TRUE, 0);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
}

//
// Denstructor
//
CExtVarFindDlg::~CExtVarFindDlg(void)
{
	g_free(m_ExportPath);
}

//
// Load catalog file
//
bool CExtVarFindDlg::LoadFile(const char *fpath, int *result)
{
	bool ok;
	char *fts_file, *pht_file;
	
	m_Phot.Close();
	m_Frame.Close();
	m_Catalog.Close();
	m_Image.Clear();
	m_SelectionList.Clear();
	m_VarFind.SetSelectionList(&m_SelectionList);

	if (m_File.Load(fpath, result)) {
		m_VarFind.SetApertures(*m_File.Apertures());
		ok = false;
		if (!ok) {
			pht_file = SetFileExtension(fpath, FILE_EXTENSION_CATALOG);
			CCatalog file;
			if (file.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_Catalog.Assign(file);
				m_PhotometryFile = false;
				m_VarFind.SetCatalogFile(&m_Catalog);
				ok = true;
			}
			g_free(pht_file);
		}
		if (!ok) {
			pht_file = SetFileExtension(fpath, "mat");
			CPhot file;
			if (file.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_Phot.Assign(file);
				m_VarFind.SetPhotometryFile(&m_Phot);
				m_PhotometryFile = true;
				ok = true;
			}
			g_free(pht_file);
		}
		if (!ok) {
			pht_file = SetFileExtension(fpath, FILE_EXTENSION_PHOTOMETRY);
			CPhot file;
			if (file.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_Phot.Assign(file);
				m_VarFind.SetPhotometryFile(&m_Phot);
				m_PhotometryFile = true;
				ok = true;
			}
			g_free(pht_file);
		}
		if (ok) {
			fts_file = SetFileExtension(fpath, FILE_EXTENSION_FITS);
			CCCDFile frame;
			if (frame.Open(fts_file, CMPACK_OPEN_READONLY)) {
				m_Frame.Assign(frame);
				if (m_Frame.GetImageData(m_Image))
					m_VarFind.SetImage(&m_Image);
			}
			g_free(fts_file);
		}
		if (ok) {
			UpdateMagDev();
			UpdateControls();
			return true;
		}
	}
	return false;
}


//
// Reload data from a file
//
void CExtVarFindDlg::ReloadFrameSet(void)
{
	int res;

	if (m_File.Load(m_Path, &res)) {
		m_VarFind.SetApertures(*m_File.Apertures());
		m_VarFind.UpdateChart();
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}


//
// Menu callback
//
void CExtVarFindDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CExtVarFindDlg *pMe = (CExtVarFindDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

//
// Command handler
//
void CExtVarFindDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_SHOW_PROPERTIES:
		ShowProperties();
		break;
	case CMD_REBUILD:
		ReloadFrameSet();
		break;
	case CMD_SAVE_CHART:
		m_VarFind.ExportChart();
		break;
	case CMD_SAVE_LCURVE:
		m_VarFind.SaveLightCurve();
		break;
	case CMD_EXPORT_LCURVE:
		m_VarFind.ExportLightCurve();
		break;
	case CMD_SAVE_MAGDEV:
		m_VarFind.SaveMagDevCurve();
		break;
	case CMD_EXPORT_MAGDEV:
		m_VarFind.ExportMagDevCurve();
		break;
	case CMD_EXPORT_DATA:
		ExportData();
		break;
	case CMD_CLOSE:
		Close();
		break;

	// View menu
	case CMD_SHOW_CHART:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_CHART);
		UpdateControls();
		break;
	case CMD_SHOW_IMAGE:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_IMAGE);
		UpdateControls();
		break;
	case CMD_SHOW_MIXED:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_MIXED);
		UpdateControls();
		break;
		
	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_FIND_VARIABLES);
		break;
	}
}

//
// VarFind callback
//
void CExtVarFindDlg::VarFindCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CExtVarFindDlg *pMe = (CExtVarFindDlg*)cb_data;

	switch (message)
	{
	case CVarFind::CB_COMPARISON_CHANGED:
		pMe->UpdateMagDev();
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_VARIABLE_CHANGED:
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_ENABLE_CTRL_QUERY:
		*((bool*)lparam) = pMe->OnEnableCtrlQuery((CVarFind::tControlId)wparam);
		break;

	case CVarFind::CB_OBJECT_VALID_QUERY:
		*((bool*)lparam) = pMe->m_File.FindObject(wparam)>=0;
		break;

	case CVarFind::CB_REMOVE_FRAMES_FROM_DATASET:
		pMe->RemoveFramesFromDataSet();
		break;
	case CVarFind::CB_REMOVE_OBJECTS_FROM_DATASET:
		pMe->RemoveObjectsFromDataSet();
		break;
	}
}

//
// Update mag-dev curve
//
void CExtVarFindDlg::UpdateMagDev(void)
{
	int res, refstar;
	CMFindProc mfind;

	const CAperture *aperture = m_VarFind.Aperture();
	if (aperture) {
		refstar = m_VarFind.ComparisonID();
		if (refstar<0)
			refstar = CMPACK_MFIND_AUTO;
		res = mfind.Init(NULL, *aperture, refstar);
		if (res==0)
			res = mfind.Execute(m_File, m_MagDev, &refstar);
		if (res==0 && refstar!=m_VarFind.ComparisonID()) 
			m_VarFind.SetComparisonStar(refstar);
	}
	m_VarFind.SetMagDev(&m_MagDev);
}

//
// Variable changed
//
void CExtVarFindDlg::UpdateLightCurve(void)
{
	int varstar, compstar;
	CSelection sel;

	varstar = m_VarFind.VariableID();
	compstar = m_VarFind.ComparisonID();
	if (varstar>=0 && compstar>=0) {
		const CAperture *aperture = m_VarFind.Aperture();
		if (aperture) {
			sel.Select(varstar, CMPACK_SELECT_VAR);
			sel.Select(compstar, CMPACK_SELECT_COMP);
			CmpackLightCurve(NULL, m_LCurve, sel, m_File, aperture->Id(), CMPACK_LCURVE_FRAME_IDS);
		}
	}
	m_VarFind.SetLightCurve(&m_LCurve);
}

//
// Enable/disable controls
//
void CExtVarFindDlg::UpdateControls(void)
{
	int varstar = m_VarFind.VariableID(), compstar = m_VarFind.ComparisonID(), 
		aperture = m_VarFind.ApertureIndex();

	m_Menu.Enable(CMD_SHOW_IMAGE, m_Frame.Valid());
	m_Menu.Enable(CMD_SHOW_MIXED, m_Frame.Valid());
	m_Menu.Check(CMD_SHOW_IMAGE, m_VarFind.DisplayMode()==CVarFind::DISPLAY_IMAGE);
	m_Menu.Check(CMD_SHOW_CHART, m_VarFind.DisplayMode()==CVarFind::DISPLAY_CHART);
	m_Menu.Check(CMD_SHOW_MIXED, m_VarFind.DisplayMode()==CVarFind::DISPLAY_MIXED);
	m_Menu.Enable(CMD_SAVE_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_SAVE_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
}

//
// Show properties
//
void CExtVarFindDlg::ShowProperties(void)
{
	CVarFindInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_File, m_Name, m_Path);
}

//
// Is this control enabled?
//
bool CExtVarFindDlg::OnEnableCtrlQuery(CVarFind::tControlId ctrl)
{
	switch (ctrl)
	{
	case CVarFind::ID_SAVE_CHART:
		return (m_PhotometryFile ? m_Phot.Valid() : m_Catalog.Valid());
	case CVarFind::ID_SAVE_LCURVE:
		return m_VarFind.VariableID()>=0 && m_VarFind.ComparisonID()>=0 && 
			m_VarFind.ApertureIndex()>=0;
	case CVarFind::ID_REMOVE_FROM_DATASET:
		return true;
	default:
		return false;
	}
}

void CExtVarFindDlg::ExportData(void)
{
	char *filename, *folder;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Export munifind data",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	// Restore last folder and file name
	folder = CConfig::GetStr("VarFind", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
	g_free(folder);
	filename = CConfig::GetStr("VarFind", "FileName", "munifind.dat");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		// Save last folder and file name
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "Folder", folder);
		g_free(folder);
		m_ExportPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "FileName", g_basename(m_ExportPath));
		gtk_widget_hide(pSaveDlg);
		// Export data in separate thread
		CProgressDlg pdlg(GTK_WINDOW(m_pDlg), "Exporting munifind data");
		pdlg.SetFileName(g_basename(m_ExportPath));
		int res = pdlg.Execute(ExportMunifindProc, this);
		if (res!=0) {
			char *msg = cmpack_formaterror(res);
			ShowError(GTK_WINDOW(m_pDlg), msg, true);
			cmpack_free(msg);
		}
		g_free(m_ExportPath);
		m_ExportPath = NULL;
	}
	gtk_widget_destroy(pSaveDlg);
}

int CExtVarFindDlg::ExportMunifindProc(CProgressDlg *sender, void *user_data)
{
	return ((CExtVarFindDlg*)user_data)->OnExportMFind(sender);
}

int CExtVarFindDlg::OnExportMFind(CProgressDlg *sender)
{
	int res = 0;
	gchar *dst_path;
	CCatalog cat2;
	CCCDFile img2;

	if (m_File.Export(m_ExportPath, 0, &res)) {
		if (m_PhotometryFile) {
			// Save reference photometry file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Phot, m_VarFind.ApertureIndex(), &res);
				cat2.Close();
			}
			g_free(dst_path);
		} else {
			// Copy catalog file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Catalog, &res);
				cat2.Close();
			}
			g_free(dst_path);
		}
		if (res==0 && m_Frame.Valid()) {
			// Save image file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_FITS);
			if (img2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				img2.Assign(m_Frame);
				img2.Close();
			}
			g_free(dst_path);
		}
	}
	return res;
}

void CExtVarFindDlg::RemoveFramesFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedFrames();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_VarFind.LightCurveData(), (intptr_t)ptr->data);
			m_File.DeleteFrame(frame);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

void CExtVarFindDlg::RemoveObjectsFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedObjects();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_VarFind.MagDevCurveData(), (intptr_t)ptr->data);
			m_File.DeleteObject(frame);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		m_VarFind.UpdateChart();
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

//---------------------------   INFO DIALOG   --------------------------------

enum tFieldId
{
	FIELD_FILE,
	FIELD_FILENAME,
	FIELD_DIRPATH
};

//
// Constructor
//
CVarFindInfoDlg::CVarFindInfoDlg(GtkWindow *pParent):CInfoDlg(pParent, 0)
{
	AddHeading(FIELD_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

//
// Show dialog
//
void CVarFindInfoDlg::ShowModal(const CFrameSet &file, const gchar *name, const gchar *path)
{
	char buf[512], *dirpath;

	// Window caption
	sprintf(buf, "%s - %s", name, "properties");
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Update properties
	SetField(FIELD_FILENAME, g_basename(path));
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	// Show dialog
	CInfoDlg::ShowModal();
}
