/**************************************************************

cmpack_graph_data.h (C-Munipack project)
Object which holds data for a graph (derived from GtkObject)
Copyright (C) 2011 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$Id: cmpack_graph_data.h,v 1.1.1.1 2012/08/12 17:00:32 dmotl Exp $

**************************************************************/
#ifndef CMPACK_GRAPH_DATA_H
#define CMPACK_GRAPH_DATA_H

#include <stdint.h>
#include <gtk/gtk.h>

#include "cmpack_widgdefs.h"

G_BEGIN_DECLS

/* Row data */
typedef struct _CmpackGraphItem {
	gdouble		x, y, error;
	CmpackColor	color;
	gchar		*tag;
	gboolean	hidden : 1;		// Display this item
	gboolean	outline : 1;	// Do not fill the circle
	gboolean	disabled : 1;	// Cannot be selected or activated
	gboolean    topmost : 1;	// Displayed on top of other items
	intptr_t	param;
} CmpackGraphItem;

/* Graph data */
typedef struct _CmpackGraphData {
	GObject		object;

	/* Private data */ 
	gint		count, capacity;	/* Number of items, allocated space */
	CmpackGraphItem	**items;			/* List of items */
} CmpackGraphData;

typedef struct {
	GObjectClass parent_class;

	/* Signals */
	void (*row_inserted)(CmpackGraphData *model, gint row);
	void (*row_updated)(CmpackGraphData *model, gint row);
	void (*row_deleted)(CmpackGraphData *model, gint row);
	void (*data_cleared)(CmpackGraphData *model);
} CmpackGraphDataClass;

#define CMPACK_TYPE_GRAPH_DATA            (cmpack_graph_data_get_type ())
#define CMPACK_GRAPH_DATA(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), CMPACK_TYPE_GRAPH_DATA, CmpackGraphData))
#define CMPACK_GRAPH_DATA_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), CMPACK_TYPE_GRAPH_DATA, CmpackGraphDataClass))
#define CMPACK_IS_GRAPH_DATA(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CMPACK_TYPE_GRAPH_DATA))
#define CMPACK_IS_GRAPH_DATA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CMPACK_TYPE_GRAPH_DATA))
#define CMPACK_GRAPH_DATA_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CMPACK_TYPE_GRAPH_DATA, CmpackGraphDataClass)) 

GType cmpack_graph_data_get_type(void);

/* Create new graph data */
CmpackGraphData *cmpack_graph_data_new(void);

/* Create new graph data */
CmpackGraphData *cmpack_graph_data_new_with_alloc(gint capacity);

/* Clear data and set number of channels and initial capacity */
void cmpack_graph_data_alloc(CmpackGraphData *data, gint capacity);

/* Get number of measurements */
gint cmpack_graph_data_nrows(CmpackGraphData *data);

/* Clear all data */
void cmpack_graph_data_clear(CmpackGraphData *data);

/* Append row to the graph */
gint cmpack_graph_data_add(CmpackGraphData *data, const CmpackGraphItem *item, gsize item_size);

/* Clear all tags */
void cmpack_graph_data_clear_tags(CmpackGraphData *data);

/* Set row tag */
void cmpack_graph_data_set_tag(CmpackGraphData *data, gint row, const gchar *tag);

/* Set topmost flag */
void cmpack_graph_data_set_topmost(CmpackGraphData *data, gint row, gboolean topmost);

/* Set color to given cell */
void cmpack_graph_data_set_color(CmpackGraphData *data, gint row, CmpackColor color);

/* Set color to given cell */
void cmpack_graph_data_set_param(CmpackGraphData *data, gint row, intptr_t param);

/* Get single value */
const CmpackGraphItem *cmpack_graph_data_get_item(CmpackGraphData *data, gint row);

/* Get row tag */
const gchar *cmpack_graph_data_get_tag(CmpackGraphData *data, gint row);

/* Get row tag */
CmpackColor cmpack_graph_data_get_color(CmpackGraphData *data, gint row);

/* Get row tag */
intptr_t cmpack_graph_data_get_param(CmpackGraphData *data, gint row);

/* Get row tag */
gint cmpack_graph_data_find_item(CmpackGraphData *data, intptr_t param);

G_END_DECLS

#endif
